#
#
#

import bond_amber

from chempy.cpv import *
from chempy import feedback

TET_TAN = 1.41
TRI_TAN = 1.732

#------------------------------------------------------------------------------
def find_known_secondary(model,anchor,known_list):
   at = model.atom[anchor]
   h_list = []
   for id in known_list:
      for b in model.bond[id]:
         atx2 = b.index[0]
         if atx2 == id:
            atx2 = b.index[1]
         if atx2 != anchor: # another bonded atom, not achor
            at2 = model.atom[atx2]
            if at2.has('coord'):
               if at2.symbol != 'H':
                  return (id,atx2)
               else:
                  h_list.append((id,atx2))
   if len(h_list): # only return hydrogen as a last resort
      return h_list[0]
   return None

#------------------------------------------------------------------------------
def simple_unknowns(model,bondfield=bond_amber):
   if feedback['actions']:
      print " "+str(__name__)+": placing unknowns..."
   # this can be used to build hydrogens and would robably work for
   # acyclic carbons as well
   if str(model.__class__) != 'chempy.models.Connected':
      raise ValueError('model is not a "Connected" model object')
   if model.nAtom:
      if not model.index:
         model.update_index()
      idx = model.index
      last_count = -1
      while 1:
         need = [ [], [], [], [] ]
         bnd_len = bondfield.length
   # find known atoms with missing neighbors, and keep track of the neighbors
         for a in model.atom:
            if a.has('coord'):
               miss = []
               know = []
               atx1 = idx[id(a)]
               bnd = model.bond[atx1]
               for b in bnd:
                  atx2 = b.index[0]
                  if atx2 == atx1:
                     atx2 = b.index[1]
                  at2 = model.atom[atx2]
                  if not at2.has('coord'):
                     miss.append(atx2)
                  else:
                     know.append(atx2)
               c = len(miss)
               if c:
                  need[c-1].append((atx1,miss,know))

         for a in need[0]: # missing only one atom
            atx1 = a[0]
            at1 = model.atom[atx1]
            atx2 = a[1][0]
            at2 = model.atom[atx2]
            know = a[2]
            if bondfield.nonlinear.has_key(at1.text_type):
               near = find_known_secondary(model,atx1,know)
               if near:
                  at3 = model.atom[near[0]]
                  if bondfield.planer.has_key(at3.text_type): # Phenolic hydrogens, etc.
                     at4 = model.atom[near[1]]
                     d1 = sub(at1.coord,at3.coord)
                     p0 = normalize(d1)
                     d2 = sub(at4.coord,at3.coord)
                     p1 = normalize(cross_product(d2,p0))
                     p2 = normalize(cross_product(p0,p1))                     
                     v = scale(p2,TRI_TAN)
                     v = normalize(add(p0,v))
                     at2.coord = add(at1.coord,scale(v,
                        bnd_len[(at1.text_type,at2.text_type)]))
                  else: # Ser, Cys, Thr hydroxyl hydrogens
                     at4 = model.atom[near[1]]
                     d2 = sub(at3.coord,at4.coord)
                     v = normalize(d2)
                     at2.coord = add(at1.coord,scale(v,
                        bnd_len[(at1.text_type,at2.text_type)]))
               elif len(know):
                  d2 = [1.0,0,0]
                  at3 = model.atom[know[0]]
                  p0 = normalize(sub(at1.coord,at3))
                  p1 = normalize(cross_product(d2,p0))
                  v = scale(p1,TET_TAN)
                  v = normalize(add(p0,v))
                  at2.coord = add(at1.coord,scale(v,
                        bnd_len[(at1.text_type,at2.text_type)]))
               else:
                  at2.coord = random_sphere(at1.coord,
                      bnd_len[(at1.text_type,at2.text_type)])
            elif len(know): # linear sum...amide, tbu, etc
               v = [0.0,0.0,0.0]
               for b in know:
                  d = sub(at1.coord,model.atom[b].coord)
                  v = add(v,normalize(d))
               v = normalize(v)
               at2.coord = add(at1.coord,scale(v,
                   bnd_len[(at1.text_type,at2.text_type)]))
            else:
               at2.coord = random_sphere(at1.coord,
                    bnd_len[(at1.text_type,at2.text_type)])

         for a in need[1]: # missing two atoms
            atx1 = a[0]
            at1 = model.atom[atx1]
            atx2 = a[1][0]
            at2 = model.atom[atx2]
            know = a[2]
            if bondfield.planer.has_key(at1.text_type): # guanido, etc
               near = find_known_secondary(model,atx1,know)
               if near: # 1-4 present
                  at3 = model.atom[near[0]]
                  at4 = model.atom[near[1]]
                  d1 = sub(at1.coord,at3.coord)
                  p0 = normalize(d1)
                  d2 = sub(at4.coord,at3.coord)
                  p1 = normalize(cross_product(d2,p0))
                  p2 = normalize(cross_product(p0,p1))
                  v = scale(p2,TRI_TAN)
                  v = normalize(add(p0,v))
                  at2.coord = add(at1.coord,scale(v,
                    bnd_len[(at1.text_type,at2.text_type)]))                                                         
                  at2 = model.atom[a[1][1]]
                  v = scale(p2,-TRI_TAN)
                  v = normalize(add(p0,v))
                  at2.coord = add(at1.coord,scale(v,
                    bnd_len[(at1.text_type,at2.text_type)]))
               elif len(know): # no 1-4 found
                  d2 = [1.0,0,0]
                  at3 = model.atom[know[0]]
                  d1 = sub(at1.coord,at3.coord)
                  p0 = normalize(d1)                  
                  p1 = normalize(cross_product(d2,p0))
                  p2 = normalize(cross_product(p0,p1))
                  v = scale(p2,TRI_TAN)
                  v = normalize(add(p0,v))
                  at2.coord = add(at1.coord,scale(v,
                    bnd_len[(at1.text_type,at2.text_type)]))
                  at2 = model.atom[a[1][1]]
                  v = scale(p2,-TRI_TAN)
                  v = normalize(add(p0,v))
                  at2.coord = add(at1.coord,scale(v,
                    bnd_len[(at1.text_type,at2.text_type)]))
               else:
                  at2.coord = random_sphere(at1.coord,
                     bnd_len[(at1.text_type,at2.text_type)])
            elif len(know)>=2: # simple tetrahedral
               at3 = model.atom[know[0]]
               at4 = model.atom[know[1]]
               v = [0.0,0.0,0.0]
               d1 = sub(at1.coord,at3.coord)
               d2 = sub(at1.coord,at4.coord)
               v = add(normalize(d1),normalize(d2))
               p0 = normalize(v)
               p1 = normalize(cross_product(d2,p0))
               v = scale(p1,TET_TAN)
               v = normalize(add(p0,v))
               at2.coord = add(at1.coord,scale(v,
                     bnd_len[(at1.text_type,at2.text_type)]))
               at2 = model.atom[a[1][1]]               
               v = scale(p1,-TET_TAN)
               v = normalize(add(p0,v))
               at2.coord = add(at1.coord,scale(v,
                     bnd_len[(at1.text_type,at2.text_type)]))
            else:
               if len(know): # sulfonamide? 
                  d2 = [1.0,0,0]
                  at3 = model.atom[know[0]]
                  d1 = sub(at1.coord,at3.coord)
                  p0 = normalize(d1)                                    
                  p1 = normalize(cross_product(d2,p0))
                  v = scale(p1,TET_TAN)
                  v = normalize(add(p0,v))
                  at2.coord = add(at1.coord,scale(v,
                     bnd_len[(at1.text_type,at2.text_type)]))
               else: # blind
                  at2.coord = random_sphere(at1.coord,
                     bnd_len[(at1.text_type,at2.text_type)])
               at4=at2
               at2=model.atom[a[1][1]]
               v = [0.0,0.0,0.0]
               d1 = sub(at1.coord,at3.coord)
               d2 = sub(at1.coord,at4.coord)
               v = add(normalize(d1),normalize(d2))
               p0 = normalize(v)
               p1 = normalize(cross_product(d2,p0))
               v = scale(p1,TET_TAN)
               v = normalize(add(p0,v))
               at2.coord = add(at1.coord,scale(v,
                  bnd_len[(at1.text_type,at2.text_type)]))

         for a in need[2]: # missing 3 atoms
            atx1 = a[0]
            at1 = model.atom[atx1]
            atx2 = a[1][0]
            at2 = model.atom[atx2]
            know = a[2]
            near = find_known_secondary(model,atx1,know)
            if near: # 1-4 present
               at3 = model.atom[near[0]]
               at4 = model.atom[near[1]]
               d1 = sub(at1.coord,at3.coord)
               p0 = normalize(d1)
               d2 = sub(at4.coord,at3.coord)
               p1 = normalize(cross_product(d2,p0))
               p2 = normalize(cross_product(p0,p1))
               v = scale(p2,-TET_TAN)
               v = normalize(add(p0,v))
               at2.coord = add(at1.coord,scale(v,
                    bnd_len[(at1.text_type,at2.text_type)]))                                                         
               at4 = at2
               at2 = model.atom[a[1][1]]
               d1 = sub(at1.coord,at3.coord)
               d2 = sub(at1.coord,at4.coord)
               v = add(normalize(d1),normalize(d2))
               p0 = normalize(v)
               p1 = normalize(cross_product(d2,p0))
               v = scale(p1,TET_TAN)
               v = normalize(add(p0,v))
               at2.coord = add(at1.coord,scale(v,
                     bnd_len[(at1.text_type,at2.text_type)]))
               at2 = model.atom[a[1][2]]               
               v = scale(p1,-TET_TAN)
               v = normalize(add(p0,v))
               at2.coord = add(at1.coord,scale(v,
                     bnd_len[(at1.text_type,at2.text_type)]))
            elif len(know): # fall-back
               d2 = [1.0,0,0]
               at3 = model.atom[know[0]]                  
               p1 = normalize(cross_product(d2,p0))
               v = scale(p1,TET_TAN)
               v = normalize(add(p0,v))
               at2.coord = add(at1.coord,scale(v,
                  bnd_len[(at1.text_type,at2.text_type)]))
               at4=at2
               at2=model.atom[a[1][1]]
               v = [0.0,0.0,0.0]
               d1 = sub(at1.coord,at3.coord)
               d2 = sub(at1.coord,at4.coord)
               v = add(normalize(d1),normalize(d2))
               p0 = normalize(v)
               p1 = normalize(cross_product(d2,p0))
               v = scale(p1,TET_TAN)
               v = normalize(add(p0,v))
               at2.coord = add(at1.coord,scale(v,
                  bnd_len[(at1.text_type,at2.text_type)]))
               at2=model.atom[a[1][2]]
               v = scale(p1,-TET_TAN)
               v = normalize(add(p0,v))
               at2.coord = add(at1.coord,scale(v,
                  bnd_len[(at1.text_type,at2.text_type)]))
            else: # worst case: add one and get rest next time around
               at2.coord=random_sphere(at2.coord,
                  bnd_len[(at1.text_type,at2.text_type)])

         for a in need[3]: # missing 4 atoms
            atx1 = a[0]
            at1 = model.atom[atx1]
            atx2 = a[1][0]
            at2 = model.atom[atx2]
            # add coordinate and get the rest next time around
            at2.coord=random_sphere(at2.coord,
                bnd_len[(at1.text_type,at2.text_type)])

         c = 0
         for a in model.atom:
            if not a.has('coord'):
               c = c + 1
         if not c:
            break;
         if c==last_count:
            break;
         last_count = c







#A* -------------------------------------------------------------------
#B* This file contains source code for the PyMOL computer program
#C* copyright 1998-2000 by Warren Lyford Delano of DeLano Scientific. 
#D* -------------------------------------------------------------------
#E* It is unlawful to modify or remove this copyright notice.
#F* -------------------------------------------------------------------
#G* Please see the accompanying LICENSE file for further information. 
#H* -------------------------------------------------------------------
#I* Additional authors of this source file include:
#-*
#-* NOTE: Based on code by John E. Grayson which was in turn 
#-* based on code written by Doug Hellmann. 
#Z* -------------------------------------------------------------------

# this section is devoted to making sure that Tkinter variables which
# correspond to Menu-displayed settings are kept synchronized with
# PyMOL

from Tkinter import *
import Pmw
from pymol import cmd
import pymol.setting
import string
import copy

class SingleEdit:
   def __init__(self,app,name,parent):
      self.app = app
      self.parent = parent
      self.name = name
      items = []

      prompt = name
      
      self.dialog = Pmw.PromptDialog(self.app.root,title=prompt,
                          buttons = ('Set', 'Cancel'),
                                   defaultbutton='Set',
                          buttonboxpos=S,
                          command = self.command)
      self.dialog.geometry("300x120")
      self.entryfield = self.dialog.component('entryfield')
      self.entry = self.entryfield.component('entry')

      txt = cmd.get_setting_text(name)
      self.entryfield.setentry(txt)
      self.entry.selection_range(0,len(txt))
      self.dialog.protocol('WM_DELETE_WINDOW',self.cancel)

      app.my_activate(self.dialog,focus=self.entry)

   def cancel(self,event=None):
      self.command(result='Done')

   def command(self,result=None):
      if result=='Set':
         st = string.strip(self.entry.get())
         if len(st):
            cmd.set(self.name,st,log=1)
         self.parent.update(self.name)
      self.app.my_deactivate(self.dialog)
            
class SetEditor:

   def __init__(self,app):

      self.app = app
      self.list = []
      for a in  pymol.setting.get_index_list():
         self.list.append("%-30s %s"%(pymol.setting._get_name(a),
                    cmd.get_setting_text(a,'',-1)))

      self.index = {}
      c = 0
      for a in pymol.setting.get_name_list():
         self.index[a] = c
         c = c + 1
         
      self.dialog = Pmw.SelectionDialog(self.app.root,title="Settings",
                          buttons = ('Edit', 'Done'),
                                   defaultbutton='Edit',
                          scrolledlist_labelpos=N,
                          label_text='Double click to edit',
                          scrolledlist_items = self.list,
                          command = self.command)
      self.dialog.geometry("500x400")
      self.listbox = self.dialog.component('scrolledlist')
      self.listbox.component('listbox').configure(font=app.my_fw_font)
      self.dialog.protocol('WM_DELETE_WINDOW',self.cancel)
      app.my_show(self.dialog)

   def cancel(self,event=None):
      self.command(result='Done')
      
   def update(self,name):
      if self.index.has_key(name):
         idx = self.index[name]
         self.listbox.delete(idx,idx)
         self.listbox.insert(idx,"%-30s %s"%(name,
                    cmd.get_setting_text(pymol.setting._get_index(name))))
         self.listbox.selection_clear()
         self.listbox.selection_set(idx)
         
   def command(self,result):
      if result=='Done':
         self.app.my_withdraw(self.dialog)
      else:
         sels = self.dialog.getcurselection()
         if len(sels)!=0:
            setting = string.strip(sels[0][0:30])
            SingleEdit(self.app,setting,self)

               




# this is quick and dirty irst crack at a gamess interface, written by someone who
# knows very little about the program (WLD) - hence the version 1 identifier...

# by the way, most of the below is untested...
      
import os
import shutil
import glob
import re
import string
import sys
import time

from chempy import feedback
from chempy.brick import Brick

atNum = {
   'H'  : 1,
   'C'  : 6,
   'N'  : 7,
   'O'  : 8,
   'F'  : 9,
   'P'  : 15,
   'S'  : 16,
   'Cl' : 17,
   'Br' : 35,
   'I'  : 53,
   }

def do(input,run_prefix=None,echo=None,
       punch=None,output=None,skip=None):
   if not run_prefix:
      run_prefix = 'gamess_run'
   if not skip:
      if feedback['gamess']:
         print " "+str(__name__)+': creating temporary files "%s.*"' % (run_prefix)
         print " "+str(__name__)+': launching gamess...' 
      try:
         for a in glob.glob(run_prefix+".*"):
            os.unlink(a)
      except:
         pass
      f = open(run_prefix+".inp",'w')
      for a in input:
         f.write(a)
      f.close()
      if echo:
         os.system(rungms_path+' '+run_prefix+" 2>&1 | tee "+run_prefix+".out")
      else:
         os.system(rungms_path+' '+run_prefix+" > "+run_prefix+".out 2>&1")      
# NFS workaround (flushes the directory cache so that glob will work)
      try: os.unlink(".sync")
      except: pass
      f = open(".sync",'w')
      f.close()
#
   if feedback['gamess']:
      print " "+str(__name__)+': job complete. '
   if punch:
      for src in glob.glob(run_prefix+".dat"):
         f = open(src)
         punch = f.readlines()
         f.close()
   if output:
      for src in glob.glob(run_prefix+".out"):
         f = open(src)
         output = f.readlines()
         f.close()
   return (output,punch)

if os.environ.has_key('GAMESS'):
   base = os.environ['GAMESS']
   bin_path = base + '/bin/'
   rungms_path = bin_path + 'rungms'
else:
   base = ''
   bin_path = ''
   params_path = ''

class State:

   def __init__(self):
      self.model = None
      self.data = None
      self.vec = None
      
   def load_model(self,model):
      self.model = model

   def get_zmat_ordering(self):
      lst = []
      for z in self.model.get_internal_tuples():
         lst.append(z[0])
      return lst
   
   def get_data_group(self,basis = None,zmat = 1):
      model = self.model
         
      gmsList = []
      
      # write header records
      gmsList.append(" $DATA\n")
      gmsList.append(model.molecule.title+" from "+str(__name__)+"\n")
      gmsList.append("C1\n")

      # write atom records in an ordering compatible with internal
      # coordinate generation
      c = 1
      for z in self.get_zmat_ordering():
         a = model.atom[z]
         if not len(a.name):
            name = a.symbol + "%02d"%c
         else:
            name = a.name
         gmsList.append("%10s %5.1f %18.10f %18.10f %18.10f\n" %
                        (name,atNum[a.symbol],a.coord[0],
                         a.coord[1],a.coord[2]))
         c = c + 1
      gmsList.append(" $END\n")
      return gmsList

   def get_ordered_data_group(self): 
      gmsList = self.data[0:3]
      flag = 1
      c = 3
      for a in self.data[3:]:
         if flag:
            flag = 0
            gmsList.append(a)
         if string.strip(a)=='':
            flag = 1
         c = c + 1
      return gmsList
   
   def get_contrl_group(self,
                        scftyp='RHF',
                        runtyp='ENERGY',
                        exetyp='RUN',
                        coord='UNIQUE',
                        nzvar = -1):
      gmsList = []
      model = self.model
      if nzvar:
         if nzvar<0:
            nzvar = (self.model.nAtom*3)-6
      gmsList.append(" $CONTRL SCFTYP=%s RUNTYP=%s EXETYP=%s\n"
                     % (scftyp,runtyp,exetyp) )
      if coord:
         gmsList.append("COORD=%s\n"%coord)
      if nzvar:
         gmsList.append("NZVAR=%d\n"%nzvar)
      chg = 0
      for a in model.atom:
         chg = chg + a.formal_charge
      chg = int(chg)
      if chg==0:
         icharg=None
      else:
         icharg='ICHARG=%d' % chg
      if icharg:
         gmsList.append("%s\n" % (icharg))
      gmsList.append(" $END\n")
      return gmsList

   def read_output_list(self,list):
      ll = len(list)
      c = 0
      crd_list = []
      chg_list = []
      nrg_list = []
      for a in list:
         if a[0:36] == ' COORDINATES OF ALL ATOMS ARE (ANGS)':
            crd_list.append(c+3)
         if a[0:13] == ' NET CHARGES:':
            chg_list.append(c+4)
         if a[0:37] == '                       TOTAL ENERGY =':
            nrg_list.append(c)
         c = c + 1
      atom = self.model.atom
      idx = {}
      c = 0
      for a in atom:
         idx[string.upper(a.name)]=c  # games converts to uppercase
         c = c + 1
      if len(crd_list):
         a = crd_list.pop()
         cc = 0
         while a<ll:
            l = list[a]
            name = string.strip(l[1:11])
            if name=='':
               break
            atom[idx[name]].coord = [float(l[16:31]),
                                     float(l[31:46]),
                                     float(l[46:61])]
            cc = cc + 1
            a = a + 1
         if cc and feedback['gamess']:
            print " "+str(__name__)+': coordinates modified for %d atoms.' % (cc)
      if len(chg_list):
         a = chg_list.pop()
         cc = 0
         while a<ll:
            l = list[a]
            name = string.strip(l[1:11])
            if name[0]=='-':
               break
            atom[idx[name]].partial_charge = float(l[19:27])
            a = a + 1
            cc = cc + 1
         if cc and feedback['gamess']:
            print " "+str(__name__)+': charges modified for %d atoms.' % (cc)
      if len(nrg_list):
         a = nrg_list.pop()
         l = list[a]
         # get energy, and convert to kcal/mole
         self.model.molecule.energy = float(string.strip(l[38:58]))*627.5095
         if feedback['gamess']:
            print " "+str(__name__)+': energy updated %12.6f.' % self.model.molecule.energy
         
   def read_punch_list(self,list):
      ll = len(list)
      c = 0
      data_list = []
      vec_list = []
      for a in list:
         if a[0:6] == ' $DATA':
            data_list.append(c)
         elif a[0:5] == ' $VEC':
            vec_list.append(c)
         c = c + 1
      if len(data_list):
         a = data_list.pop()
         self.data = []
         data = self.data
         while a<ll:
            la = list[a]
            data.append(la)
            if la[0:5] == ' $END':
               break
            a = a + 1
         if feedback['gamess']:
            print " "+str(__name__)+': read $DATA group.'
      if len(vec_list):
         a = vec_list.pop()
         self.vec = []
         vec = self.data
         while a<ll:
            la = list[a]
            vec.append(la)
            if la[0:5] == ' $END':
               break
            a = a + 1
         if feedback['gamess']:
            print " "+str(__name__)+': read new $VEC group.'

   def update_data_coords(self): # update coordinates of ordered atoms in $DATA
      idx = {}
      c = 0
      for a in self.model.atom:
         idx[string.upper(a.name)]=c
         c = c + 1
      if self.data:
         flag = 1
         c = 3
         for a in self.data[3:]:
            if flag:
               flag = 0
               kee = a[0:3]
               if not idx.has_key(kee):
                  break
               i = idx[kee]
               at = self.model.atom[i]
               self.data[c]="%-10s%5.1f%18.10f%18.10f%18.10f\n" % (
                  at.name,atNum[at.symbol],at.coord[0],
                  at.coord[1],at.coord[2])
            if string.strip(a)=='':
               flag = 1
            c = c + 1
   
   def read_density_list(self,list,brick,z_step):
      ll = len(list)
      c = 0
      den_list = []
      for a in list:
         if a[0:37] == ' ELECTRON DENSITY, IPOINT,X,Y,Z,EDENS':
            den_list.append(c+1)
         c = c + 1
      if len(den_list):
         lst = 0
         a = den_list.pop()
         for x in xrange(brick.dim[0]):
            for y in xrange(brick.dim[1]):
               brick.lvl[x][y][z_step] = float(list[a][36:51])
               a = a + 1 
         if feedback['gamess']:
            print " "+str(__name__)+': read density slice %d of %d.' %(
               z_step+1,brick.dim[2])

   def read_potential_list(self,list,brick,z_step):
      ll = len(list)
      c = 0
      pot_list = []
      for a in list:
         if a[0:51] == 'THE ROWS OF THE ELECTROSTATIC POTENTIAL GRID (A.U.)':
            pot_list.append(c+1)
         c = c + 1
      if len(pot_list):
         lst = 0
         a = pot_list.pop()
         for x in xrange(brick.dim[0]):
            aa = a
            mat = list[aa][0:3]
            col = []
            while 1:
               if list[aa][0:3]==mat:
                  col.append(list[aa][5:])
                  aa = aa + 1
               else:
                  break
            a = aa
            vst = string.split(string.strip(string.join(col)))
            for y in xrange(brick.dim[1]):
               brick.lvl[x][y][z_step] = float(vst[y])
         if feedback['gamess']:
            print " "+str(__name__)+': read potential slice %d of %d.' %(
               z_step+1,brick.dim[2])

   def get_basis_group(self,gbasis='N31',ngauss=6,ndfunc=1):
      gmsList = []
      gmsList.append(" $BASIS GBASIS=%s NGAUSS=%d NDFUNC=%d\n" %
                     (gbasis,ngauss,ndfunc))
      model = self.model
      chg = 0
      for a in model.atom:
         chg = chg + a.formal_charge
      chg = int(chg)
      if chg<0:
         diffsp=' DIFFSP=.TRUE.'
      else:
         diffsp=None
      if diffsp:
         gmsList.append("%s\n" % (diffsp))
      gmsList.append(" $END\n")
      return gmsList

   def get_zmat_ext_freeze_torsion(self,flag=3):
      # requires PYMOL to read dihedrals from structure
      # requires list of dihedrals from tinker.amber
      #
      from pymol import cmd
      from tinker.amber import Topology      

      cmd.load_model(self.model,'_gamess1')
      model = self.model

      # get mapping of model ordering to zmat ordering
      m2z = {}
      z2m = {}
      c = 1 # GAMESS is one-based
      for a in self.get_zmat_ordering():
         m2z[a] = c
         z2m[c] = a
         c = c + 1

      # get all torsions in the molecule

      topo = Topology(self.model)

      # find those where flag is set in all atoms

      mask = 2 ** flag

      frozen_list = []

      for a in topo.torsion.keys():
         if (model.atom[a[0]].flags&
             model.atom[a[1]].flags&
             model.atom[a[2]].flags&
             model.atom[a[3]].flags)&mask:
            frozen_list.append(a)

      print " freeze-torsion: %d torsions will be frozen."%len(frozen_list)

      irzmat = []
      ifzmat = []
      fvalue = []
      if len(frozen_list):

         for frozen in frozen_list:
            # find additional torsions which need to be removed

            remove = []

            for a in topo.torsion.keys():
               if (((a[1]==frozen[1])and(a[2]==frozen[2])) or
                   ((a[2]==frozen[1])and(a[1]==frozen[2]))):
                  if a!=frozen:
                     remove.append(a)

            # convert to internal coordinate ordering

            frozen_z = (m2z[frozen[0]],m2z[frozen[1]],
                        m2z[frozen[2]],m2z[frozen[3]])

            remove_z = []
            for a in remove:
               remove_z.append(m2z[a[0]],m2z[a[1]],m2z[a[2]],m2z[a[3]])

            # now reorder atoms in torsions to reflect z_matrix ordering
            # (not sure this is necessary)

            if frozen_z[0]>frozen_z[3]:
               frozen_z = (frozen_z[3],frozen_z[2],frozen_z[1],frozen_z[0])
            tmp_z = []
            for a in remove_z:
               if a[0]>a[3]:
                  tmp_z.append((a[3],a[2],a[1],a[0]))
               else:
                  tmp_z.append(a)
            remove_z = tmp_z

            # get value of the fixed torsion

            fixed = (z2m[frozen_z[0]],z2m[frozen_z[1]],
                     z2m[frozen_z[2]],z2m[frozen_z[3]])

            dihe = cmd.get_dihedral("(_gamess1 and id %d)"%fixed[0],
                             "(_gamess1 and id %d)"%fixed[1],
                             "(_gamess1 and id %d)"%fixed[2],
                             "(_gamess1 and id %d)"%fixed[3])

            # write out report for user edification

            print " freeze-torsion: fixing freeze-torsion:"
            print " freeze-torsion: %d-%d-%d-%d (pymol), %d-%d-%d-%d (gamess)"%(
               fixed[0],fixed[1],fixed[2],fixed[3],
               frozen_z[0],frozen_z[1],frozen_z[2],frozen_z[3])
            print " freeze-torsion: at %5.3f"%dihe
            print " freeze-torsion: removing redundant torsions:"
            for a in remove_z[1:]:
               print " freeze-torsion: %d-%d-%d-%d (pymol), %d-%d-%d-%d (gamess)"%(
                  z2m[a[0]],z2m[a[1]],z2m[a[2]],z2m[a[3]],
                  a[0],a[1],a[2],a[3])

            # add parameters for this torsion into the list

            ifzmat.append((3,frozen_z[0],frozen_z[1],frozen_z[2],frozen_z[3]))
            fvalue.append(dihe)

            if len(remove_z):
               for a in remove_z[1:]:
                  irzmat.append((3,a[0],a[1],a[2],a[3]))

      # generate restrained dihedral information

      zmat_ext = []
      if len(ifzmat):
         zmat_ext.append(" IFZMAT(1)=\n")
         comma = ""
         for a in ifzmat:
            zmat_ext.append(comma+"%d,%d,%d,%d,%d\n"%a)
            comma = ","
      if len(fvalue):
         zmat_ext.append(" FVALUE(1)=\n")
         comma = ""
         for a in fvalue:
            zmat_ext.append(comma+"%1.7f\n"%a)
            comma = ","
      if len(irzmat):
         zmat_ext.append(" IRZMAT(1)=\n")
         comma = ""
         for a in irzmat:
            zmat_ext.append(comma+"%d,%d,%d,%d,%d\n"%a)
            comma = ","

      cmd.delete("_gamess1") # important
      if len(zmat_ext):
         return zmat_ext
      else:
         return None
      
   def get_zmat_group(self,auto=1,dlc=1,zmat_extend=None):
      gmsList = []
      if auto and dlc:
         if zmat_extend == None:
            gmsList.append(" $ZMAT DLC=.TRUE. AUTO=.TRUE. $END\n")
         else:
            gmsList.append(" $ZMAT DLC=.TRUE. AUTO=.TRUE.\n")
            gmsList.extend(zmat_extend)
            gmsList.append(" $END\n")            
      else:
         raise RuntimeError
      return gmsList
   
   def get_eldens_group(self,morb=0):
      gmsList = []
      gmsList.append(" $ELDENS IEDEN=1 MORB=%i \n" % morb)
      gmsList.append("WHERE=GRID OUTPUT=PUNCH\n $END\n")
      return gmsList

   def get_elpot_group(self,morb=0):
      gmsList = []
      gmsList.append(" $ELPOT IEPOT=1 \n")
      gmsList.append("WHERE=GRID OUTPUT=PUNCH\n $END\n")
      return gmsList

   def get_guess_group(self,guess='HUCKEL'):
      return [" $GUESS GUESS=%s $END\n"%guess]
   
   def get_grid_group(self,brick,z_step):
      origin = (
         brick.origin[0],
         brick.origin[1],
         brick.origin[2]+brick.grid[2]*z_step)
      x_coord = (
         brick.origin[0]+brick.range[0]+brick.grid[0]/100.0,
         brick.origin[1],
         brick.origin[2]+brick.grid[2]*z_step)
      y_coord = (
         brick.origin[0],
         brick.origin[1]+brick.range[1]+brick.grid[1]/100.0,
         brick.origin[2]+brick.grid[2]*z_step)
      gmsList = [
         " $GRID ORIGIN(1)=%12.5f,%12.5f,%12.5f\n" % origin,
         "XVEC(1) = %12.5f,%12.5f,%12.5f\n" % x_coord,
         "YVEC(1) = %12.5f,%12.5f,%12.5f\n" % y_coord,
         "SIZE = %12.5f\n" % brick.grid[0],
         " $END\n"
         ]
      return gmsList
   
   def get_scf(self,dirscf=1):
      gmsList = []
      if dirscf:
         gmsList.append(" $SCF DIRSCF=.TRUE. $END\n")
      return gmsList

   def get_optimize_job(self,dirscf=1,zmat_extend=None):
      gmsList = []
      gmsList.extend(self.get_contrl_group(runtyp='OPTIMIZE'))
      gmsList.extend(self.get_basis_group())
      gmsList.extend(self.get_scf(dirscf=dirscf))
      gmsList.extend(self.get_data_group())
      gmsList.extend(self.get_zmat_group(zmat_extend=zmat_extend))
      gmsList.append(" $STATPT NSTEP=50 $END\n")
      return gmsList
         
   def get_optimize_charge_job(self):
      gmsList = self.get_optimize_job()
      gmsList.append(" $ELPOT IEPOT=1 WHERE=PDC $END\n")
      gmsList.append(" $PDC PTSEL=GEODESIC CONSTR=CHARGE $END\n")
      return gmsList

   def get_energy_charge_job(self):
      gmsList = self.get_energy_job()
      gmsList.append(" $ELPOT IEPOT=1 WHERE=PDC $END\n")
      gmsList.append(" $PDC PTSEL=GEODESIC CONSTR=CHARGE $END\n")
      return gmsList
   
   def get_prop_job(self):
      gmsList = []
      gmsList.extend(self.get_contrl_group(runtyp = 'PROP',
                                           nzvar=0))
      gmsList.extend(self.get_guess_group(guess='MOREAD'))
      self.update_data_coords()
      gmsList.extend(self.data)
      gmsList.extend(self.vec)
      return gmsList
      
   def get_energy_job(self):
      gmsList=[]
      gmsList.extend(self.get_contrl_group(
         runtyp = 'ENERGY'
         ))
      gmsList.extend(self.get_basis_group())
      gmsList.extend(self.get_scf())
      gmsList.extend(self.get_data_group())
      gmsList.extend(self.get_zmat_group())
      return gmsList

   def get_density_job(self,brick,z_step,morb=0):
      gmsList = self.get_prop_job()
      gmsList.extend(self.get_eldens_group(morb=morb))
      gmsList.extend(self.get_grid_group(brick,z_step))
      return gmsList

   def get_potential_job(self,brick,z_step):
      gmsList = self.get_prop_job()
      gmsList.extend(self.get_elpot_group())
      gmsList.extend(self.get_grid_group(brick,z_step))
      return gmsList

   def get_prop_charge_job(self):
      gmsList = self.get_prop_job()
      gmsList.append(" $ELPOT IEPOT=1 WHERE=PDC $END\n")
      gmsList.append(" $PDC PTSEL=GEODESIC CONSTR=CHARGE $END\n")
      return gmsList

   def get_density(self,brick,morb=0,run_prefix=None):
      for a in xrange(brick.dim[2]):
         gmsList = self.get_density_job(brick,a,morb=morb)
         result = do(gmsList,punch=1,
                     run_prefix=run_prefix)
         self.read_density_list(result[1],brick,a)

   def get_potential(self,brick,run_prefix=None):
      for a in xrange(brick.dim[2]):
         gmsList = self.get_potential_job(brick,a)
         result = do(gmsList,punch=1,
                     run_prefix=run_prefix)
         self.read_potential_list(result[1],brick,a)

   def get_charges(self,run_prefix=None):
      gmsList = self.get_energy_job()
      result = do(gmsList,output=1,punch=1,
                  run_prefix=run_prefix)
      self.read_output_list(result[0])
      self.read_punch_list(result[1])

   def get_energy(self,run_prefix=None):
      gmsList = self.get_energy_job()
      result = do(gmsList,output=1,punch=1,
                  run_prefix=run_prefix)
      self.read_output_list(result[0])
      self.read_punch_list(result[1])

   def get_optimized_energy(self,run_prefix=None,zmat_extend=None):
      gmsList = self.get_optimize_job(zmat_extend=zmat_extend)
      result = do(gmsList,output=1,punch=1,
                  run_prefix=run_prefix)
      self.read_output_list(result[0])
      self.read_punch_list(result[1])

   def get_optimized_charges(self,run_prefix=None,skip=None):
      gmsList = self.get_optimize_charge_job()
      result = do(gmsList,output=1,punch=1,
                  run_prefix=run_prefix,skip=skip)
      self.read_output_list(result[0])
      self.read_punch_list(result[1])

   def get_prop_charges(self,run_prefix=None):
      gmsList = self.get_prop_charge_job()
      result = do(gmsList,output=1,punch=1,
                  run_prefix=run_prefix)
      self.read_output_list(result[0])
      self.read_punch_list(result[1])


if os.environ.has_key('GAMESS'):
   base = os.environ['GAMESS']
   rungms_path = base + '/rungms'
else:
   base = ''
   rungms_path = ''









import pymol
import cmd
import setting
import parsing

QuietException = parsing.QuietException

tmp_editor = "_tmp_editor"
tmp_ed_save = "_tmp_ed_save"
tpk1 = "_tmp_tpk1"
tpk2 = "_tmp_tpk2"

# routines to assist in molecular editing

def attach_fragment(selection,fragment,hydrogen,anchor):
   if not selection in cmd.get_names("selections"):
      if fragment in cmd.get_names("objects"):
         print " Error: an object with than name already exists"
         raise QuietException
      else:
         cmd.fragment(fragment)
         if cmd.get_setting_legacy("auto_remove_hydrogens"):
            cmd.remove("(hydro and %s)"%fragment)
   else:
      cmd.fragment(fragment,tmp_editor)
      if cmd.count_atoms("((%s) and elem h)"%selection,quiet=1):
         cmd.fuse("(%s and id %d)"%(tmp_editor,hydrogen),"(pk1)",1)
         if cmd.get_setting_legacy("auto_remove_hydrogens"):
            cmd.remove("(hydro and pkchain)")            
      else:
         cmd.remove("(%s and id %d)"%(tmp_editor,hydrogen))
         cmd.fuse("(%s and id %d)"%(tmp_editor,anchor),"(pk1)",1)
         if cmd.get_setting_legacy("auto_remove_hydrogens"):
            cmd.remove("(hydro and pkchain)")            
      cmd.delete(tmp_editor)
      
def attach_amino_acid(selection,amino_acid):
   if not selection in cmd.get_names("selections"):
      if amino_acid in cmd.get_names("objects"):
         print " Error: an object with than name already exists"
         raise QuietException
      cmd.fragment(amino_acid)
      if cmd.get_setting_legacy("auto_remove_hydrogens"):
         cmd.remove("(hydro and %s)"%amino_acid)
      if cmd.count_atoms("((%s) and name c)"%amino_acid,quiet=1):
         cmd.edit("((%s) and name c)"%amino_acid)
   else:
      ss = int(cmd.get_setting_legacy("secondary_structure"))
      if ss:
         if ss==1: # helix
            phi=-57.0
            psi=-47.0
         elif ss==2: # antipara-beta
            phi=-139.0
            psi=135.0
         elif ss==3: # para-beta
            phi=-119.0
            psi=113.0
         else:
            phi=180.0
            psi=180.0

      cmd.fragment(amino_acid,tmp_editor)
      if cmd.count_atoms("((%s) and elem n)"%selection,quiet=1):
         cmd.select(tmp_ed_save,"(%s)"%selection)
         cmd.iterate("(%s)"%selection,"stored.resv=resv")
         pymol.stored.resi = str(pymol.stored.resv-1)
         cmd.alter(tmp_editor,"resi=stored.resi")
         cmd.fuse("(%s and name C)"%(tmp_editor),"(pk1)",2)
         if cmd.get_setting_legacy("auto_remove_hydrogens"):
            cmd.remove("(pkchain and hydro)")
         cmd.set_dihedral("(name ca and neighbor pk2)","(pk2)","(pk1)","(name ca,ch3 and neighbor pk1)",180.0)
         cmd.set_geometry("pk2",3,3) # make nitrogen planer
         if ss:
            cmd.select(tpk1,"pk2")
            cmd.select(tpk2,"pk1")
            if amino_acid[0:3]!='pro':
               cmd.set_dihedral( # PHI
                  "(name c and neighbor (name ca and neighbor "+tpk1+"))", # C
                  "(name ca and neighbor "+tpk1+")", # CA 
                  tpk1, # N
                  tpk2, # C
                  phi)
            cmd.set_dihedral( # PSI (n-1)
               tpk1, # N
               tpk2, # C
               "(name ca and neighbor "+tpk2+")", # CA
               "(name n and neighbor (name ca and neighbor "+tpk2+"))", # C
               psi)
            cmd.delete(tpk1)
            cmd.delete(tpk2)               
         sele = ("(name N and (byres neighbor %s) and not (byres %s))"%
                 (tmp_ed_save,tmp_ed_save))
         if cmd.count_atoms(sele,quiet=1):
            cmd.edit(sele)
         cmd.delete(tmp_ed_save)
               
      elif cmd.count_atoms("((%s) and elem c)"%selection,quiet=1):
         cmd.select(tmp_ed_save,"(%s)"%selection)
         cmd.iterate("(%s)"%selection,"stored.resv=resv")
         pymol.stored.resi = str(pymol.stored.resv+1)
         cmd.alter(tmp_editor,"resi=stored.resi")
         cmd.fuse("(%s and name N)"%(tmp_editor),"(pk1)",2)
         if cmd.get_setting_legacy("auto_remove_hydrogens"):
            cmd.remove("(pkchain and hydro)")
         cmd.set_dihedral("(name ca and neighbor pk2)","(pk2)","(pk1)","(name ca,ch3 and neighbor pk1)",180.0)
         cmd.set_geometry("pk1",3,3) # make nitrogen planer
         if ss:
            cmd.select(tpk1,"pk1")
            cmd.select(tpk2,"pk2")
            if amino_acid[0:3]!='pro':
               cmd.set_dihedral( # PHI
                  tpk2, # C
                  tpk1, # N
                  "(name ca and neighbor "+tpk1+")", # CA 
                  "(name c and neighbor (name ca and neighbor "+tpk1+"))", # C
                  phi)
            cmd.set_dihedral( # PSI (n-1)
               "(name n and neighbor (name ca and neighbor "+tpk2+"))", # C
               "(name ca and neighbor "+tpk2+")", # CA
               tpk2, # C
               tpk1, # N
               psi)
            cmd.delete(tpk1)
            cmd.delete(tpk2)               
         sele = ("(name C and (byres neighbor %s) and not (byres %s))"%
                 (tmp_ed_save,tmp_ed_save))
         if cmd.count_atoms(sele,quiet=1):
            cmd.edit(sele)
         cmd.delete(tmp_ed_save)
      elif cmd.count_atoms("((%s) and elem h)"%selection,quiet=1):
         print " Error: please pick a nitrogen or carbonyl carbon to grow from."
         cmd.delete(tmp_editor)
         raise QuietException
     
   cmd.delete(tmp_editor)
      





#A* -------------------------------------------------------------------
#B* This file contains source code for the PyMOL computer program
#C* copyright 1998-2000 by Warren Lyford Delano of DeLano Scientific. 
#D* -------------------------------------------------------------------
#E* It is unlawful to modify or remove this copyright notice.
#F* -------------------------------------------------------------------
#G* Please see the accompanying LICENSE file for further information. 
#H* -------------------------------------------------------------------
#I* Additional authors of this source file include:
#-* 
#-* 
#-*
#Z* -------------------------------------------------------------------

import string
from chempy import cpv
#import popen2
import os
from pymol import cmd

POINTS             = 0.0
LINES              = 1.0
LINE_LOOP          = 2.0
LINE_STRIP         = 3.0
TRIANGLES          = 4.0
TRIANGLE_STRIP     = 5.0
TRIANGLE_FAN       = 6.0
#QUADS              = 7.0
#QUAD_STRIP         = 8.0
#POLYGON            = 9.0                                                            

STOP               =  0.0
NULL               =  1.0
BEGIN              =  2.0
END                =  3.0
VERTEX             =  4.0
NORMAL             =  5.0
COLOR              =  6.0
SPHERE             =  7.0
TRIANGLE           =  8.0
CYLINDER           =  9.0
LINEWIDTH          = 10.0
WIDTHSCALE         = 11.0
ENABLE             = 12.0
DISABLE            = 13.0
SAUSAGE            = 14.0
CUSTOM_CYLINDER    = 15.0

LIGHTING           = float(0x0B50)

def molauto(*arg):
   name = "mols"
   sele = "(all)"
   marg = "-nice"
   la = len(arg)
   if la:
      name = arg[0]
   if la>1:
      sele = arg[1]
   if la>2:
      marg = arg[2]
   cmd.save("molauto.pdb",sele)
   print "molauto %s -nocentre molauto.pdb | molscript -r > molauto.r3d"%marg
   os.system("molauto %s -nocentre molauto.pdb | molscript -r > molauto.r3d"%marg)
   f = open("molauto.r3d")
   rr = RenderReader(f)
   f.close()
   cmd.load_cgo(rr.obj,name)

# the following implementation causes full-blown system crashes on some machines.
#   (stdout,stdin) = popen2.popen2("molauto %s -nocentre molauto.pdb | molscript -r > molauto.r3d"%marg)
#
#   if stdin:
#      stdin.close()
#      rr = RenderReader(stdout)
#      cmd.load_cgo(rr.obj,name)

def measure_text(font,text,
                 axes=[[1.0,0.0,0.0],[0.0,1.0,0.0],[0.0,0.0,1.0]]):
   w = 0
   x = axes[0]
   for char in text:
      if font.has_key(char):
         w = w + font[char][0]*x[0]
   return w

def wire_text(cgo,font,pos,text,
              axes = [[1.0,0.0,0.0],[0.0,1.0,0.0],[0.0,0.0,1.0]]): # modifies pos
   x = axes[0]
   y = axes[1]
   for char in text:
      if font.has_key(char):
         fc = font[char]
         stroke = 0
         w = fc[0]
         f = fc[1]
         c = 0
         l = len(f)-2
         while c<l:
            if not f[c]:
               if stroke: cgo.append(END)
               cgo.append(BEGIN)
               cgo.append(LINE_STRIP)
               stroke = 1
            ax = f[c+1]
            ay = f[c+2]
            cgo.append(VERTEX)
            cgo.append(pos[0]+x[0]*ax+y[0]*ay)
            cgo.append(pos[1]+x[1]*ax+y[1]*ay)
            cgo.append(pos[2]+x[2]*ax+y[2]*ay)         
            c = c + 3
         pos[0] = pos[0] + w*x[0]
         pos[1] = pos[1] + w*x[1]
         pos[2] = pos[2] + w*x[2]
         if stroke: cgo.append(END)

def cyl_text(cgo,font,pos,text,radius=0.1,color=[1.0,1.0,1.0],
              axes = [[1.0,0.0,0.0],[0.0,1.0,0.0],[0.0,0.0,1.0]]): # modifies pos
   x = axes[0]
   y = axes[1]
   for char in text:
      if font.has_key(char):
         fc = font[char]
         stroke = 0
         w = fc[0]
         f = fc[1]
         c = 0
         l = len(f)-2
         while c<l:
            ax = f[c+1]
            ay = f[c+2]
            next = [(pos[0]+x[0]*ax+y[0]*ay),
                    (pos[1]+x[1]*ax+y[1]*ay),
                    (pos[2]+x[2]*ax+y[2]*ay)]
            if f[c]:
               if stroke:
                  cgo.append(SAUSAGE)
                  cgo.extend(last)
                  cgo.extend(next)
                  cgo.append(radius)
                  cgo.extend(color)
                  cgo.extend(color)
            else:
               stroke = 1
            last = next
            c = c + 3
         pos[0] = pos[0] + w*x[0]
         pos[1] = pos[1] + w*x[1]
         pos[2] = pos[2] + w*x[2]


def from_r3d(fname):
   result = None
   input = open(fname)
   if input:
      rr = RenderReader(input)
      result = rr.obj
   return result

class RenderReader:

   def append_last(self):
      if self.app_fn:
         apply(self.app_fn)
         self.app_fn=None
      
   def append_tri(self):
      if self.l_vert and not self.l_norm:
         d0 = cpv.sub(self.l_vert[0],self.l_vert[1])
         d1 = cpv.sub(self.l_vert[0],self.l_vert[2])
         n0 = cpv.cross_product(d0,d1)
         n0 = cpv.normalize_failsafe(n0)
         n1 = [-n0[0],-n0[1],-n0[2]]
         ns = cpv.scale(n0,0.002)
         if not self.tri_flag:
            self.obj.append(BEGIN)
            self.obj.append(TRIANGLES)
            self.tri_flag = 1
         self.obj.append(COLOR)  # assuming unicolor
         self.obj.extend(self.t_colr[0])
         self.obj.append(NORMAL)
         self.obj.extend(n0)
         self.obj.append(VERTEX)
         self.obj.extend(cpv.add(self.l_vert[0],ns))
         self.obj.append(COLOR)  # assuming unicolor
         self.obj.extend(self.t_colr[1])
         self.obj.append(NORMAL)
         self.obj.extend(n0)
         self.obj.append(VERTEX)
         self.obj.extend(cpv.add(self.l_vert[1],ns))
         self.obj.append(COLOR)  # assuming unicolor
         self.obj.extend(self.t_colr[2])
         self.obj.append(NORMAL)
         self.obj.extend(n0)
         self.obj.append(VERTEX)
         self.obj.extend(cpv.add(self.l_vert[2],ns))
         self.obj.append(COLOR)  # assuming unicolor
         self.obj.extend(self.t_colr[0])
         self.obj.append(NORMAL)
         self.obj.extend(n1)
         self.obj.append(VERTEX)
         self.obj.extend(cpv.sub(self.l_vert[0],ns))
         self.obj.append(COLOR)  # assuming unicolor
         self.obj.extend(self.t_colr[1])
         self.obj.append(NORMAL)
         self.obj.extend(n1)
         self.obj.append(VERTEX)
         self.obj.extend(cpv.sub(self.l_vert[1],ns))
         self.obj.append(COLOR)  # assuming unicolor
         self.obj.extend(self.t_colr[2])
         self.obj.append(NORMAL)
         self.obj.extend(n1)
         self.obj.append(VERTEX)
         self.obj.extend(cpv.sub(self.l_vert[2],ns))

      elif self.l_vert and self.t_colr and self.l_norm:
         if not self.tri_flag:
            self.obj.append(BEGIN)
            self.obj.append(TRIANGLES)
            self.tri_flag = 1
         self.obj.append(COLOR) # assuming unicolor
         self.obj.extend(self.t_colr[0])
         self.obj.append(NORMAL)
         self.obj.extend(self.l_norm[0])
         self.obj.append(VERTEX)
         self.obj.extend(self.l_vert[0])
         self.obj.append(COLOR) # assuming unicolor
         self.obj.extend(self.t_colr[1])
         self.obj.append(NORMAL)
         self.obj.extend(self.l_norm[1])
         self.obj.append(VERTEX)
         self.obj.extend(self.l_vert[1])
         self.obj.append(COLOR) # assuming unicolor
         self.obj.extend(self.t_colr[2])
         self.obj.append(NORMAL)
         self.obj.extend(self.l_norm[2])
         self.obj.append(VERTEX)
         self.obj.extend(self.l_vert[2])
      self.l_vert=None
      self.t_colr=None
      self.l_norm=None

   def append_cyl(self):
      if self.l_vert and self.c_colr and self.l_radi:
         if self.tri_flag:
            self.tri_flag=0
            self.obj.append(END)
         self.obj.append(SAUSAGE)
         d = cpv.sub(self.l_vert[1],self.l_vert[0])
         d = cpv.normalize_failsafe(d)
         d0 = cpv.scale(d,self.l_radi/4.0)
         self.obj.extend(cpv.add(self.l_vert[0],d0))
         self.obj.extend(cpv.sub(self.l_vert[1],d0))
         self.obj.append(self.l_radi)
         self.obj.extend(self.c_colr[0])
         self.obj.extend(self.c_colr[1])
      self.l_vert=None
      self.c_colr=None
      self.l_radi=None

   def tri(self,f):
      self.append_last()
      l = f.readline()
      if l:
         self.app_fn=self.append_tri
         s = string.split(l)
         self.l_vert = [[float(s[0]),float(s[1]),float(s[2])],
                  [float(s[3]),float(s[4]),float(s[5])],
                  [float(s[6]),float(s[7]),float(s[8])]]
         self.t_colr_t = [float(s[9]),float(s[10]),float(s[11])]
         self.t_colr = [self.t_colr_t,self.t_colr_t,self.t_colr_t]

   def tri_normal(self,f):
      l = f.readline()
      if l:
         s = string.split(l)
         self.l_norm = [[float(s[0]),float(s[1]),float(s[2])],
                  [float(s[3]),float(s[4]),float(s[5])],
                  [float(s[6]),float(s[7]),float(s[8])]]

   def cyl(self,f):
      self.append_last()
      l = f.readline()
      if l:
         self.app_fn = self.append_cyl
         s = string.split(l)
         self.l_vert = [[float(s[0]),float(s[1]),float(s[2])],
                  [float(s[4]),float(s[5]),float(s[6])]]
         self.l_radi = float(s[3])
         self.c_colr_t = [float(s[8]),float(s[9]),float(s[10])]
         self.c_colr = [self.c_colr_t,self.c_colr_t]

   def sphere(self,f):
      self.append_last()
      l = f.readline()
      if l:
         s = string.split(l)
         self.obj.append(COLOR)
         self.obj.extend([float(s[4]),float(s[5]),float(s[6])])
         self.obj.append(SPHERE)
         self.obj.extend([float(s[0]),float(s[1]),float(s[2]),float(s[3])])


   def __init__(self,input):
      # Author: Warren DeLano
      # Modifications: Robert Campbell
      self.app_fn = None
      self.l_vert = None
      self.t_colr = None
      self.c_colr = None
      self.l_radi = None
      self.l_norm = None
      self.o_vert = None
      self.tri_flag = 0
      self.cc = 0
      self.obj = []
      for a in range(20):
         input.readline()
      dispatch = [
         None,
         self.tri,
         self.sphere,
         self.cyl,
         None,
         self.cyl,
         None,
         self.tri_normal,
         ]
      ld = len(dispatch)
      while 1:
         l = input.readline()
         if not l:
            break
         if l[0] != '#':
            v = string.split(l)
            n=int(v[0])
            if(n<ld):
               dd = dispatch[n]
               if dd:
                  apply(dd,(input,))
               else:
                  # skip over lines that don't match desired object type
                  input.readline()
            elif ( n != 9 ):
               # don't read another line if render object type 0
               input.readline()
      self.append_last()
      if self.tri_flag:
         self.obj.append(END)
      input.close()



#A* -------------------------------------------------------------------
#B* This file contains source code for the PyMOL computer program
#C* copyright 1998-2000 by Warren Lyford Delano of DeLano Scientific. 
#D* -------------------------------------------------------------------
#E* It is unlawful to modify or remove this copyright notice.
#F* -------------------------------------------------------------------
#G* Please see the accompanying LICENSE file for further information. 
#H* -------------------------------------------------------------------
#I* Additional authors of this source file include:
#-* 
#-* 
#-*
#Z* -------------------------------------------------------------------

import os
import pymol
import string
import parsing
import threading

from glob import glob
from cmd import _cmd,lock,unlock,Shortcut,QuietException
from cmd import _feedback,fb_module,fb_mask

def cd(dir):
   '''
DESCRIPTION

   "cd" changes the current working directory.

USAGE
   
   cd <path>

SEE ALSO

   pwd, ls, system
   '''
   dir = os.path.expanduser(dir)
   dir = os.path.expandvars(dir)
   os.chdir(dir)  # raises on error
   return 1

def pwd():
   '''
DESCRIPTION

   Print current working directory.

USAGE
   
   pwd

SEE ALSO

   cd, ls, system
   '''
   print os.getcwd()
   return 1

def ls(pattern=None):
   '''
DESCRIPTION

   List contents of the current working directory.

USAGE
   
   ls [pattern]
   dir [pattern]

EXAMPLES

   ls
   ls *.pml

SEE ALSO

   cd, pwd, system   
   '''
   if pattern==None:
      pattern = "*"
   else:
      pattern = os.path.expanduser(pattern)
      pattern = os.path.expandvars(pattern)
   if string.find("*",pattern)<0:
      lst = glob(pattern+"/*")
   else:
      lst = []
   if not len(lst):
      lst = glob(pattern)
   if len(lst):
      lst.sort()
      lst = parsing.list_to_str_list(lst)
      for a in lst:
         print a
   else:
      print " ls: Nothing found.  Is that a valid path?"
   return 1

def system(command,async=0):
   '''
DESCRIPTION

   "system" executes a command in a subshell under Unix or Windows.

USAGE

   system command 

PYMOL API

   cmd.system(string command,int sync=1)

NOTES

   async can only be specified from the Python level (not the command language)
   
   if async is 0 (default), then the result code from "system" is returned in r

   if async is 1, then the command is run in a separate thread whose object is
   returned
   
SEE ALSO

   ls, cd, pwd
   '''
   if async:
      r = threading.Thread(target=_cmd.system,args=(str(command),1))
      r.start()
   else:
      r = _cmd.system(str(command),0)
   return r # special meaning

def paste(): # INTERNAL
   r=1
   lst = []
   if hasattr(pymol,"machine_get_clipboard"):
      lst = pymol.machine_get_clipboard()
   if len(lst):
      while 1:
         if len(lst[-1]):
            if ord(lst[-1][-1])>32: # trim off final CR
               break;
         else:
            break;
         lst[-1]=lst[-1][:-1]
      _cmd.paste(lst)      
   return r 




class Map:
   
   def __init__(self):
      self.cell_dim = [ 1.0, 1.0, 1.0]
      self.cell_ang = [ 90.0, 90.0, 90.0 ]
      self.cell_div = [ 10, 10, 10 ] # how many divisions per cell axis
      self.first = [ 0, 0, 0 ] # first slice on each axis
      self.last = [ 9, 9, 9 ] # last slice on each axis
      self.format = 'Empty'
      
   def from_c_object(self,cobj,format,cell_dim,cell_ang,cell_div,first,last):
      # pack with defined types
      self.format = str(format)
      self.c_object = cobj
      self.cell_ang = [ float(cell_ang[0]),
                        float(cell_ang[1]),
                        float(cell_ang[2])]
      self.cell_dim = [ float(cell_dim[0]),
                        float(cell_dim[1]),
                        float(cell_dim[2])]
      self.cell_div = [ int(cell_div[0]),
                        int(cell_div[1]),
                        int(cell_div[2])]
      self.first = [ int(first[0]),
                     int(first[1]),
                     int(first[2])]
      self.last = [ int(last[0]),
                    int(last[1]),
                    int(last[2])]
      





class Wizard:

   def __init__(self):
      self.menu = {}
      self.prompt = None
      self.panel = None
      
   def get_prompt(self):
      return self.prompt

   def get_panel(self):
      return self.panel

   def do_pick(self,bondFlag):
      pass

   def cleanup(self):
      pass

   def get_menu(self,tag):
      result = None
      if self.menu.has_key(tag):
         result = self.menu[tag]
      return result




#A* -------------------------------------------------------------------
#B* This file contains source code for the PyMOL computer program
#C* copyright 1998-2000 by Warren Lyford Delano of DeLano Scientific. 
#D* -------------------------------------------------------------------
#E* It is unlawful to modify or remove this copyright notice.
#F* -------------------------------------------------------------------
#G* Please see the accompanying LICENSE file for further information. 
#H* -------------------------------------------------------------------
#I* Additional authors of this source file include:
#-* Peter Haebel, Byron DeLaBarre
#-* 
#-*
#Z* -------------------------------------------------------------------

import cmd
import glob
import math

def load(*args):
   nam = "mov"
   if len(args)>1:
      nam = args[1]
   fils = glob.glob(args[0])
   fils.sort()
   if not len(fils):
      print "Error: no matching files"
   else:
      for a in fils:
         cmd.load(a,nam)

def rock(first,last,angle=30,phase=0,loop=1,axis='y'):
   first=int(first)
   last=int(last)
   angle=float(angle)
   phase=float(phase)
   loop=int(loop)
   nstep = (last-first)+1
   if nstep<0:
      nstep = 1
   if loop:
      subdiv = nstep
   else:
      subdiv = nstep+1
   ang_cur = math.pi*phase/180
   ang_inc = 2*math.pi/subdiv
   ang_cur = ang_cur - ang_inc
   a = 0
   while a<nstep:
      last = angle*math.sin(ang_cur)/2
      ang_cur = ang_cur + ang_inc
      disp = angle*math.sin(ang_cur)/2
      diff = disp-last
      # com = "mdo %d:turn %s,%8.3f" % (first+a,axis,diff)
      # cmd.do(com)
      cmd.mdo("%d"%(first+a),"turn %s,%8.3f"% (axis,diff))      
      a = a + 1

def roll(first,last,loop=1,axis='y'):
   first=int(first)
   last=int(last)
   loop=int(loop)
   n = last - first
   if loop:
      step = 2*math.pi/(n+1)
   else:
      step = 2*math.pi/n   
   a = 0
   deg = (180*step/math.pi)
   while a<=n:
      # com = "mdo %d:turn %s,%8.3f" % (first+a,axis,deg)
      # cmd.do(com)
      cmd.mdo("%d" % (first+a), "turn %s,%8.3f" % (axis,deg))
      a = a + 1

def tdroll(first,rangex,rangey,rangez,skip=1):
   '''
AUTHOR

   Byron DeLaBarre

USAGE

   movie.tdroll(rangx,rangey,rangez,skip=1,mset=0)
   
   rangex/y/z = rotation range on respective axis
   enter 0 for no rotation.

   skip is angle increment in each frame
   
   Use skip to reduce final movie size or to speed up rotation.
   
EXAMPLE

   movie.tdroll 360,360,360,5
   
'''
   rangex = float(rangex)
   rangey = float(rangey)
   rangez = float(rangez)
   skip = int(skip)
   axis=['x','y','z']
   rangel=[rangex,rangey,rangez]
   axpos=0
   frpos=1
   for ax in axis:
      range = int(rangel[axpos])
      if range:
         leftover = divmod(range,skip)
         print leftover[1]
         if leftover[1]:
           range = range + int(leftover[1])
         a = 0
         while a<range:
            cmd.mdo("%d" % (first+frpos-1), "turn %s,%8.3f" % (ax,skip))
            a = a + skip
            frpos = frpos + 1
         axpos = axpos + 1
      else:
         axpos = axpos + 1
   print (" tdroll: defined rotations for", frpos - 1,
          "frames, starting at frame %d"%first)

def zoom(first,last,step=1,loop=1,axis='z'):
   # Author: Peter Haebel
   first=int(first)
   last=int(last)
   step=int(step)
   loop=int(loop)
   n = last - first
   a = 0
   while a<=n:
      if (loop and a>n/2):
         s = -step
      else:
         s = step
      # com = "mdo %d:move %s,%8.3f" % (first+a,axis,s)
      # cmd.do(com)
      cmd.mdo("%d" % (first+a),"move %s,%8.3f" % (axis,s))
      a = a + 1

def nutate(first,last,angle=30,phase=0,loop=1,shift=math.pi/2.0,factor=0.01):
   first=int(first)
   last=int(last)
   angle=float(angle)
   phase=float(phase)
   loop=int(loop)
   nstep = (last-first)+1
   if nstep<0:
      nstep = 1
   if loop:
      subdiv = nstep
   else:
      subdiv = nstep+1
   ang_cur = math.pi*phase/180
   ang_inc = 2*math.pi/subdiv
   ang_cur = ang_cur - ang_inc
   a = 0
   while a<nstep:
      lastx = angle*math.sin(ang_cur)/2
      lasty = angle*math.sin(ang_cur+shift)/2
      ang_cur = ang_cur + ang_inc
      nextx = angle*math.sin(ang_cur)/2
      nexty = angle*math.sin(ang_cur+shift)/2      
      # com = "mdo %d:turn %s,%8.3f" % (first+a,axis,diff)
      # cmd.do(com)
      cmd.mdo("%d"%(first+a),"turn x,%8.3f;turn y,%8.3f;turn y,%8.3f;turn x,%8.3f"%
              (-lastx,-lasty,nexty,nextx))
      a = a + 1

def screw(first,last,step=1,angle=30,phase=0,loop=1,axis='y'):
   # Author: Peter Haebel
   first=int(first)
   last=int(last)
   step=int(step)
   angle=float(angle)
   phase=float(phase)
   loop=int(loop)
   nstep = (last-first)+1
   if nstep<0:
      nstep = 1
   if loop:
      subdiv = nstep
   else:
      subdiv = nstep+1
   ang_cur = math.pi*phase/180
   ang_inc = 2*math.pi/subdiv
   ang_cur = ang_cur - ang_inc
   a = 0
   while a<nstep:
      if (loop and a>=nstep/2):
         s = -step
      else:
         s = step
      last = angle*math.sin(ang_cur)/2
      ang_cur = ang_cur + ang_inc
      disp = angle*math.sin(ang_cur)/2
      diff = disp-last
      # com = "mdo %d:turn %s,%8.3f; move z,%8.3f" % (first+a,axis,diff,s)
      # cmd.do(com)
      cmd.mdo("%d" % (first+a), "turn %s,%8.3f; move z,%8.3f" % (axis,diff,s))
      a = a + 1




#A* -------------------------------------------------------------------
#B* This file contains source code for the PyMOL computer program
#C* copyright 1998-2000 by Warren Lyford Delano of DeLano Scientific. 
#D* -------------------------------------------------------------------
#E* It is unlawful to modify or remove this copyright notice.
#F* -------------------------------------------------------------------
#G* Please see the accompanying LICENSE file for further information. 
#H* -------------------------------------------------------------------
#I* Additional authors of this source file include:
#-* 
#-* 
#-*
#Z* -------------------------------------------------------------------

# parser2.py
# An improved command parser for PyMOL, but still a terrible kluuge
# PyMOL needs to migrate to a real parser soon!

# === Goals:
#  1. improved 1to1 mapping between pymol "cmd" API and command language
#  2. support for named arguments
#  3. support for calling arbitrary python functions via this mapping

# === Syntatic Examples

# * simple commands
# command

# * commands with arguments
#            
# command value1
# command value1,value2
# command value1,value2,value3

# * commands with named arguments
#
# command argument1=value1
# command argument1=value1,argument2=value2,argument3=value3
# command argument3=value1,argument2=value2,argument1=value1   

# * mixed...
#
# command value1,argument3=value3,arg

# * commands with legacy '=' support for first argument
#
# command string1=value1    
# * which should map to
# command string1,value1

# * legacy '=' combined as above
#
# command string1=value1,value2
# command string1=value1,value2,value3,...
# command string1=value1,argument3=value3

# === Burdens placed on API functions...
#
# None. However, function must have real arguments for error checking.
# 

import re
import string
import sys
import threading
import new

class QuietException:
   def __init__(self,args=None):
      self.args = args

# constants for keyword modes

SIMPLE      = 0  # original pymol parsing (deprecated)
SINGLE      = 1  # ignore ";", treat entire line as a single command
RUN         = 2  # run command 
SPAWN       = 3  # for spawn and fork commands
ABORT       = 4  # terminates command script
NO_CHECK    = 10 # no error checking 
STRICT      = 11 # strict name->argument checking
LEGACY      = 12 # support legacy construct str1=val1,... -> str1,val1,...
LITERAL     = 20 # argument is to be treated as a literal string 
LITERAL1    = 21 # one regular argument, followed by literal string
LITERAL2    = 22 # two regular argument, followed by literal string

# key regular expressions

command_re = re.compile(r"[^\s]+")
whitesp_re = re.compile(r"\s+")
comma_re = re.compile(r"\s*,\s*")
arg_name_re = re.compile(r"[A-Za-z0-9_]+\s*\=")
nester_char_re = re.compile(r"\(|\)|\[|\]")
nester_re = re.compile(r"[^,;]*[\(\[]")
arg_pre_nester_re = re.compile(r"([^,;\(\[]+)[\(\[]")
arg_post_nester_re = re.compile(r"[^,;\(\[]*")
arg_easy_nester_re = re.compile(r"\([^,]*\)|\[[^,]*\]")
arg_hard_nester_re = re.compile(r"\(.*\)|\[.*\]")
# NOTE '''sdf'sdfs''' doesn't work in below.
arg_value_re = re.compile(r"'''[^']*'''|'[^']*'|"+r'"[^"]*"|[^,;]+')
def trim_nester(st):
   # utility routine, returns single instance of a nested string
   # should be modified to handle quotes too                  
   pc = 1
   l = len(st)
   c = 1
   while c<l:
      if st[c] in ('(','['):
         pc = pc + 1
      if st[c] in (')',']'):
         pc = pc - 1
      c = c + 1
      if not pc:
         break
   if pc:
      return None
   return st[0:c]

def parse_arg(st,mode=STRICT):
   '''
parse_arg(st)

expects entire command to be passed in

returns list of tuples of strings: [(None,value),(name,value)...]
'''
   result = [] 
   # current character
   cc = 0
   mo = command_re.match(st[cc:])
   if mo:
      cc=cc+mo.end(0)
      while 1:
         if mode>=LITERAL: # LITERAL argument handling
            if (mode-LITERAL)==len(result):
               result.append((None,string.strip(st[cc:])))
               return result
         # clean whitespace
         mo = whitesp_re.match(st[cc:])
         if mo:
            cc=cc+mo.end(0)            
         if not len(st[cc:]):
            break
         # read argument name, if any         
         mo = arg_name_re.match(st[cc:])
         if mo:
            nam = string.strip(mo.group(0)[:-1])
            cc=cc+mo.end(0)
         else:
            nam = None
         # clean whitespace
         mo = whitesp_re.match(st[cc:])
         if mo:
            cc=cc+mo.end(0)
         # is one or more nesters present?
         skip_flag = 0
         if nester_re.match(st[cc:]):
            skip_flag = 1
            nest_flag = 1
            nest_str = ''
            while nest_flag: # parse all the nesters
               nest_flag = 0
               # text before nester?
               mo = arg_pre_nester_re.match(st[cc:])
               if mo:
                  nest_str = nest_str + mo.group(1)
                  cc=cc+mo.end(1)
               # special handling for nesters (selections, lists, tuples, etc.)
               mo = arg_easy_nester_re.match(st[cc:]) # no internal commas
               if mo:
                  cnt = len(nester_char_re.findall(mo.group(0))) 
                  if (2*(cnt/2))!=cnt: # make sure nesters are matched in count
                     mo = None
               if mo:
                  nest_str = nest_str + mo.group(0)
                  cc=cc+mo.end(0)
                  # text after nester?
                  mo = arg_post_nester_re.match(st[cc:])
                  if mo:
                     post_nester = mo.group(0)
                     cc=cc+mo.end(0)
                  nest_str = nest_str + post_nester
                  nest_flag = 1 # one more cycle
               else:
                  mo = arg_hard_nester_re.match(st[cc:])
                  if mo:
                     se = trim_nester(mo.group(0))
                     if se==None:
                        print "Error: "+st
                        print "Error: "+" "*cc+"^ syntax error (type 1)."
                        raise QuietException
                     else:
                        cc = cc + len(se)
                        nest_str = nest_str + se
                        # text after nester?
                        mo = arg_post_nester_re.match(st[cc:])
                        if mo:
                           nest_str = nest_str + mo.group(0)
                           cc=cc+mo.end(0)
                        nest_flag = 1 # one more cycle
            if not len(nest_str): # we must have failed to parse...
               skip_flag = 0
            else:
               result.append((nam,string.strip(nest_str)))
         if not skip_flag:
            # no nester, so just read normal argument value
            argval = None
            mo = arg_value_re.match(st[cc:])
            if not mo:
               print "Error: "+st
               print "Error: "+" "*cc+"^ syntax error (type 2)."
               raise QuietException
            argval = mo.group(0)
            cc=cc+mo.end(0)
            while 1: # pickup unqouted characters after quotes
               mo = arg_value_re.match(st[cc:])
               if not mo:
                  break
               argval = argval + mo.group(0)
               cc=cc+mo.end(0)
            if argval!=None:
               result.append((nam,string.strip(argval)))
         # clean whitespace
         mo = whitesp_re.match(st[cc:])
         if mo:
            cc=cc+mo.end(0)
         # skip over comma
         if len(st[cc:]):
            mo = comma_re.match(st[cc:])
            if mo:
               cc=cc+mo.end(0)
            else:
               print "Error: "+st
               print "Error: "+" "*cc+"^ syntax error (type 3)."
               raise QuietException
   if __name__!='__main__':
      if cmd._feedback(cmd.fb_module.parser,cmd.fb_mask.debugging):
         cmd.fb_debug.write(" parsing-DEBUG: tup: "+str(result)+"\n")
   return result

def dump_str_list(list):
   lst = list_to_str_list(list)
   for a in lst:
      print a

def list_to_str_list(list,width=77,margin=2): # format strings into a list
   result = []
   ll=len(list)
   if ll>0:
      mxln = 1
      for a in list:
         if len(a)>mxln:
            mxln = len(a)
      n_col = width/mxln
      width = width - margin
      while (n_col * mxln + n_col*2)>width:
         n_col = n_col - 1
      if n_col < 1:
         n_col = 1
      ll = len(list)
      n_row = len(list)/n_col
      while (n_row*n_col)<ll:
         n_row = n_row + 1
      rows = []
      for a in range(n_row):
         rows.append([])
      row = 0
      pad_list = []
      for a in list:
         pad_list.append(("%-"+str(mxln)+"s")%a)
      for a in pad_list:
         rows[row].append(a)
         row = row + 1
         if row >= n_row:
            row = 0
      for a in rows:
         st = margin*' '
         row = 0
         st = st + string.join(a,'  ')
         result.append(st)
   return result

def dump_arg(name,arg_lst,nreq):
   ac = 0
   pc = 0
   st = "Usage: "+name
   for a in arg_lst:
      if ac>=nreq:
         st = st + " ["
         pc = pc + 1
      if ac:
         st = st + ", " + a
      else:
         st = st + " " + a
      ac = ac + 1
   print st,"]"*pc
   
def prepare_call(fn,lst,mode=STRICT,name=None): # returns tuple of arg,kw or excepts if error
   if name==None:
      name=fn.__name__
   result = (None,None)
   arg = []
   kw = {}
   co = fn.func_code
   if (co.co_flags & 0xC): # disable error checking for *arg or **kw functions
      mode = NO_CHECK
   arg_nam = co.co_varnames[0:co.co_argcount]
   narg = len(arg_nam)
   if fn.func_defaults:
      ndef = len(fn.func_defaults)
   else:
      ndef = 0
   nreq = narg-ndef
   if len(lst)==1:
      if lst[0]==(None,'?'):
         dump_arg(name,arg_nam,nreq)         
         raise QuietException
      
   if mode==NO_CHECK:
      # no error checking
      for a in lst:
         if a[0]==None:
            arg.append(a[1])
         else:
            kw[a[0]]=a[1]
      # set feedback argument (quiet), if extant, results enabled, and not overridden
      if "quiet" in arg_nam:
         if not kw.has_key("quiet"):
            if __name__!='__main__':
               if cmd._feedback(cmd.fb_module.cmd,cmd.fb_mask.results):
                  kw["quiet"] = 0
   else:
      # error checking enabled

      # build name dictionary, with required flag
      arg_dct={}
      c = 0
      for a in arg_nam:
         arg_dct[a]=c<nreq
         c = c + 1
      if mode==LEGACY:
         # handle legacy string=value transformation
         tmp_lst = []
         for a in lst:
            if(a[0]!=None):
               if not arg_dct.has_key(a[0]):
                  tmp_lst.extend([(None,a[0]),(None,a[1])])
               else:
                  tmp_lst.append(a)
            else:
               tmp_lst.append(a)
         lst = tmp_lst
      # make sure we don't have too many arguments
      if len(lst)>narg:
         if not narg:
            print "Error: too many arguments for %s; None expected."%(name)
         elif narg==nreq:
            print "Error: too many arguments for %s; %d expected, %d found."%(
               name,nreq,len(lst))
            dump_arg(name,arg_nam,nreq)
         else:
            print "Error: too many arguments for %s; %d to %d expected, %d found."%(
               name,nreq,narg,len(lst))
            dump_arg(name,arg_nam,nreq)            
         raise QuietException
      # match names to unnamed arguments to create argument dictionary
      ac = 0
      val_dct = {}
      for a in lst:
         if a[0]==None:
            if ac>=narg:
               print "Parsing-Error: ambiguous argument: '"+str(a[1])+"'"
               raise QuietException
            else:
               val_dct[arg_nam[ac]]=a[1]
         else:
            val_dct[a[0]]=a[1]
         ac = ac + 1
      # now check to make sure we don't have any missing arguments
      for a in arg_nam:
         if arg_dct[a]:
            if not val_dct.has_key(a):
               print "Parsing-Error: missing required argument:",a
               raise QuietException
      # return all arguments as keyword arguments
      kw = val_dct
      # set feedback argument (quiet), if extant, results enabled, and not overridden
      if arg_dct.has_key("quiet"):
         if not kw.has_key("quiet"):
            if cmd._feedback(cmd.fb_module.cmd,cmd.fb_mask.results):
               kw["quiet"] = 0
   if __name__!='__main__':
      if cmd._feedback(cmd.fb_module.parser,cmd.fb_mask.debugging):
         cmd.fb_debug.write(" parsing-DEBUG: kw: "+str(kw)+"\n")      
   return (arg,kw)


# launching routines

def run_as_module(file,spawn=0):
   name = re.sub('[^A-Za-z0-9]','_',file)
   mod = new.module(name)
   mod.__file__ = file
   sys.modules[name]=mod
   if spawn:
      t = threading.Thread(target=execfile,
         args=(file,mod.__dict__,mod.__dict__))
      t.setDaemon(1)
      t.start()
   else:
      execfile(file,mod.__dict__,mod.__dict__)
      del sys.modules[name]
      del mod

def run_as_thread(args,global_ns,local_ns):
   t = threading.Thread(target=execfile,args=(args,global_ns,local_ns))
   t.setDaemon(1)
   t.start()
               
def split(*arg): # custom split-and-trim
   '''
split(string,token[,count]) -> list of strings
 
UTILITY FUNCTION, NOT PART OF THE API
Breaks strings up by tokens but preserves quoted strings and
parenthetical groups (such as atom selections).

USAGE OF THIS FUNCTION IS DISCOURAGED - THE GOAL IS TO
MAKE IT UNNECESSARY BY IMPROVING THE BUILT-IN PARSER
'''
   str = arg[0]
   tok = arg[1]
   if len(arg)>2:
      mx=arg[2]
   else:
      mx=0
   pair = { '(':')','[':']','{':'}',"'":"'",'"':'"' }
   plst = pair.keys()
   stack = []
   lst = []
   c = 0
   nf = 0
   l = len(str)
   wd = ""
   while str[c]==tok:
      c = c + 1
   while c<l:
      ch = str[c]
      if (ch in tok) and (len(stack)==0):
         lst.append(string.strip(wd))
         nf = nf + 1
         if mx:
            if nf==mx:
               wd = string.strip(str[c+1:])
               break;
         wd = ''
         w = 0
      else:
         if len(stack):
            if ch==stack[0]:
               stack = stack[1:]
            elif (ch in plst):
               stack[:0]=[pair[ch]]
         elif (ch in plst):
            stack[:0]=[pair[ch]]
         wd = wd + ch
      c = c + 1
   if len(wd):
      lst.append(string.strip(wd))
   return lst
   

if __name__=='__main__':

# regular expressions

   mo = arg_name_re.match("earth=testing,hello")
   tv = mo.group(0)
   print tv == "earth=",tv
   
   mo = arg_value_re.match("testing,hello")
   tv = mo.group(0)
   print tv == "testing",tv

   mo = arg_value_re.match("'testing,\"hello'")
   tv = mo.group(0)
   print tv == "'testing,\"hello'",tv

   mo = arg_value_re.match('"testing,\'hello"')
   tv = mo.group(0)
   print tv == '"testing,\'hello"',tv

   mo = arg_value_re.match("\'\'\'testing,h\"ello\'\'\'")
   tv = mo.group(0)
   print tv=="'''testing,h\"ello'''",tv



# argument parsing

   tv = parse_arg("command val")
   print tv==[(None, 'val')],tv
   tv = parse_arg("command val1,val2")
   print tv==[(None, 'val1'), (None, 'val2')],tv
   tv=parse_arg("command val1,val2,val3")
   print tv== [(None, 'val1'), (None, 'val2'), (None, 'val3')],tv      

   tv = parse_arg("command arg=val")
   print tv == [('arg', 'val')],tv
   tv = parse_arg("command arg1=val1,arg2=val2")
   print tv == [('arg1', 'val1'), ('arg2', 'val2')],tv
   tv = parse_arg("command val1,val2,val3")
   print tv == [(None, 'val1'), (None, 'val2'), (None, 'val3')],tv

   tv = parse_arg("command arg=val")
   print tv == [('arg', 'val')] ,tv
   tv = parse_arg("command arg_1=val1,arg2=val2")
   print tv == [('arg_1', 'val1'), ('arg2', 'val2')],tv
   tv = parse_arg("command val1,val2,val3")
   print tv == [(None, 'val1'), (None, 'val2'), (None, 'val3')],tv
   
   tv = parse_arg("command val1, str2=val2, str3=val3")
   print tv == [(None, 'val1'), ('str2', 'val2'), ('str3', 'val3')],tv
   tv = parse_arg("command val1, str2 = val2,str3= val3")
   print tv == [(None, 'val1'), ('str2', 'val2'), ('str3', 'val3')],tv
   tv = parse_arg("command val1, str2 =val2 ,str3 = val3")
   print tv == [(None, 'val1'), ('str2', 'val2'), ('str3', 'val3')],tv
   tv = parse_arg("command val1, str2 =val2 , str3= val3   ")
   print tv == [(None, 'val1'), ('str2', 'val2'), ('str3', 'val3')],tv
   
   tv = parse_arg("command multi word 1, str2 =multi word 2")
   print tv == [(None, 'multi word 1'), ('str2', 'multi word 2')],tv

   tv = parse_arg("command multi word 1  , str2 =  multi word 2 ")
   print tv == [(None, 'multi word 1'), ('str2', 'multi word 2')],tv

   tv = parse_arg("command ( name;ca,c,n ), sel1= (name c,n) ")
   print tv == [(None, '( name;ca,c,n )'), ('sel1', '(name c,n)')],tv

   tv = parse_arg("command ( byres (name;ca,c,n) ), sel1= (name c,n) ")
   print tv==[(None, '( byres (name;ca,c,n) )'), ('sel1', '(name c,n)')],tv

   tv = parse_arg("command ( byres (name;ca,c,n ), sel1= (name c,n)) ")
   print tv==[(None, '( byres (name;ca,c,n ), sel1= (name c,n))')],tv

   tv = parse_arg("command test,")
   print tv==[(None,'test')],tv

   tv = parse_arg("command =sdf,") # desired behavior?
   print tv==[(None,'=sdf')],tv

   tv = parse_arg("command 'hello\",bob','''hel\"lo,dad'''") 
   print tv==[(None, '\'hello",bob\''), (None, '\'\'\'hel"lo,dad\'\'\'')],tv

   tv = parse_arg("command \"'hello''bob'\"")
   print tv==[(None, '"\'hello\'\'bob\'"')],tv


   tv = parse_arg("command this,is,a command;b command",mode=LITERAL)
   print tv==[(None, 'this,is,a command;b command')], tv

   tv = parse_arg("command this,a command;b command",mode=LITERAL1)
   print tv==[(None, 'this'), (None, 'a command;b command')], tv

   tv = parse_arg("command this,is,a command;b command",mode=LITERAL2)
   print tv==[(None, 'this'), (None, 'is'), (None, 'a command;b command')],tv

   tv = parse_arg("command this,is,a=hello;b command",mode=LITERAL2)
   print tv==[(None, 'this'), (None, 'is'), (None, 'a=hello;b command')],tv
   
# expected exceptions

   try:
      tv = parse_arg("command ( byres (name;ca,c,n ), sel1= (name c,n) ")      
      print 0, "exception missed"
   except QuietException:
      print 1, "exception raised (as expected)"

   try:
      tv = parse_arg("command ,")
      print 0, "exception missed"
   except QuietException:
      print 1, "exception raised (as expected)"

   try:
      tv = parse_arg("command berf=,")
      print 0, "exception missed"
   except QuietException:
      print 1, "exception raised (as expected)"

   try:
      tv = parse_arg("command 'hello''bob'")
      print 0, "exception missed"
   except QuietException:
      print 1, "exception raised"

# function call preparation

   def fn1(req1,req2,opt1=1,opt2=2):
      pass

   tv = prepare_call(fn1,parse_arg("dummy hello,world"))
   print tv==([], {'req2': 'world', 'req1': 'hello'}),tv

   tv = prepare_call(fn1,parse_arg("dummy hello,world,opt2=hi"))
   print tv==([], {'opt2': 'hi', 'req2': 'world', 'req1': 'hello'}),tv

   tv = prepare_call(fn1,parse_arg("dummy req1=hello,req2=world,opt2=hi")) 
   print tv==([], {'opt2': 'hi', 'req2': 'world', 'req1': 'hello'}),tv

   tv = prepare_call(fn1,parse_arg("dummy req2=world,req1=hello,opt2=hi")) 
   print tv==([], {'opt2': 'hi', 'req2': 'world', 'req1': 'hello'}),tv

   tv = prepare_call(fn1,parse_arg("dummy hello,world,give,tea"))
   print tv==([], {'opt2': 'tea', 'req2': 'world', 'req1': 'hello', 'opt1': 'give'}),tv

   tv = prepare_call(fn1,parse_arg("dummy hello,world,give,tea"),10)
   print tv==(['hello', 'world', 'give', 'tea'], {}),tv

   tv = prepare_call(fn1,parse_arg("dummy hello=world,give,tea"),12) # test legacy support
   print tv==([], {'opt2': 'tea', 'req2': 'world', 'req1': 'hello', 'opt1': 'give'}),tv

   def fn2(*arg):
      pass

   tv = prepare_call(fn2,parse_arg("dummy req1=hello,req2=world,opt2=hi")) 
   print tv==([], {'opt2': 'hi', 'req2': 'world', 'req1': 'hello'}),tv

   def fn3(*arg,**kw):
      pass

   tv = prepare_call(fn3,parse_arg("dummy req1=hello,req2=world,opt2=hi")) 
   print tv==([], {'opt2': 'hi', 'req2': 'world', 'req1': 'hello'}),tv

   def fn4(req1,req2,**kw):
      pass

   tv = prepare_call(fn4,parse_arg("dummy req1=hello,req2=world,opt2=hi")) 
   print tv==([], {'opt2': 'hi', 'req2': 'world', 'req1': 'hello'}),tv

   tv = prepare_call(fn4,parse_arg("dummy req1=hi and (r; 10) and r. 10,req2=world,opt2=hi")) 
   print tv==([], {'opt2': 'hi', 'req2': 'world', 'req1': 'hi and (r; 10) and r. 10'}),tv

   tv = prepare_call(fn4,parse_arg("dummy req1=hi and (r. 10,12) and r. 10,req2=world,opt2=hi")) 
   print tv==([], {'opt2': 'hi', 'req2': 'world', 'req1': 'hi and (r. 10,12) and r. 10'}),tv

   tv = prepare_call(fn4,parse_arg("dummy req1=hi & (r. 10,12) & (r;10),req2=world,opt2=hi")) 
   print tv==([], {'opt2': 'hi', 'req2': 'world', 'req1': 'hi & (r. 10,12) & (r;10)'}),tv

   tv = prepare_call(fn4,parse_arg("dummy req1=hi & (r. 10,12) & (r;10) & resi 10,req2=world,opt2=hi")) 
   print tv==([], {'opt2': 'hi', 'req2': 'world', 'req1': 'hi & (r. 10,12) & (r;10) & resi 10'}),tv

   tv = prepare_call(fn4,parse_arg("dummy req1=(r. 10,12) & (r;10) & resi 10,req2=world,opt2=hi")) 
   print tv==([], {'opt2': 'hi', 'req2': 'world', 'req1': '(r. 10,12) & (r;10) & resi 10'}),tv

#   tv = list_to_str_list(['hello','world','this-long-string','hi','dude'])
#   print tv
else:
   import cmd




from pymol.wizard import Wizard
from pymol import cmd
import pymol
import copy

default_map = [ '', '', '']
default_level = [ 1.0, 3.0, 1.0]
default_radius = 8.0
default_track = 0

class Density(Wizard):

   def __init__(self):

      Wizard.__init__(self)
      
      # mode selection subsystem
      
      self.radius = default_radius
      self.map = copy.deepcopy(default_map)
      self.level = copy.deepcopy(default_level)
      self.track = copy.deepcopy(default_track)
      self.avail_maps = []
      
      self.menu['radius'] = [
                             [1, '4.0 A Radius','cmd.get_wizard().set_radius(4)'],
                             [1, '5.0 A Radius','cmd.get_wizard().set_radius(5)'],
                             [1, '6.0 A Radius','cmd.get_wizard().set_radius(6)'],
                             [1, '8.0 A Radius','cmd.get_wizard().set_radius(8)'],
                             [1, '10.0 A Radius','cmd.get_wizard().set_radius(10)'],
                             [1, '15.0 A Radius','cmd.get_wizard().set_radius(15)'],
                             [1, '20.0 A Radius','cmd.get_wizard().set_radius(20)'],
                             [1, '50.0 A Radius','cmd.get_wizard().set_radius(50)'],
                             ]
                              
      self.menu['map0'] = []
      self.menu['map1'] = []
      self.menu['map2'] = []

      level_menu = lambda x:[ [1, '1.0 sigma','cmd.get_wizard().set_level(%d,1.0)'%x],
                              [1, '1.5 sigma','cmd.get_wizard().set_level(%d,1.5)'%x],
                              [1, '2.0 sigma','cmd.get_wizard().set_level(%d,2.0)'%x],
                              [1, '3.0 sigma','cmd.get_wizard().set_level(%d,3.0)'%x],
                              [1, '5.0 sigma','cmd.get_wizard().set_level(%d,5.0)'%x]]
      
      self.menu['level0'] = level_menu(0)
      self.menu['level1'] = level_menu(1)
      self.menu['level2'] = level_menu(2)
      
      self.menu['track'] = [
      [ 1, "Track & Zoom", 'cmd.get_wizard().set_track(0)'],
      [ 1, "Track & Set Origin", 'cmd.get_wizard().set_track(1)'],
      [ 1, "Track Off", 'cmd.get_wizard().set_track(2)'],
      ]

      if (self.map[0] == '') and (self.map[1] == '') and (self.map[2]==''):
         for a in cmd.get_names(): # automatically load first map we find
            if cmd.get_type(a)=='object:map':
               self.map[0]=a
               break
      self.update_map_menus()
      
   def update_map_menus(self):

      self.avail_maps = []
      
      for a in cmd.get_names('objects'):
         if cmd.get_type(a)=='object:map':
            self.avail_maps.append(a)

      c = 0
      for a in self.map:
         map_kee = 'map'+str(c)
         level_kee = 'level'+str(c)
         self.menu[map_kee] = [[2,'Select Map','']]
         for a in self.avail_maps:
            self.menu[map_kee].append([ 1,a,'cmd.get_wizard().set_map(%d,"%s")'%(c,a) ])
         self.menu[map_kee].append([1,'(none)','cmd.get_wizard().set_map(%d,"")'%c])
         c = c + 1

   def set_track(self,track):
      self.track = track
      cmd.refresh_wizard()
      
   def set_level(self,map,level):
      self.level[map] = level
      self.update_maps()
      cmd.refresh_wizard()

   def set_map(self,map,map_name):
      self.map[map] = map_name
      cmd.refresh_wizard()

   def set_radius(self,radius):
      self.radius = radius
      self.update_maps()
      cmd.refresh_wizard()

   def update_maps(self):
      if 'pk1' in cmd.get_names('selections'):
         save = cmd.get_setting_text('auto_zoom')
         save = cmd.set('auto_zoom',0,quiet=1)                     
         c = 0
         for a in self.map:
            oname = 'w'+str(c+1)+'_'+a
            if oname not in cmd.get_names():
               color = 1
            else:
               color = 0
            if len(a):
               if cmd.get_type(a)=='object:map':
                  cmd.isomesh(oname,a,self.level[c],
                              "(pk1)",self.radius,state=1)
               if color:
                  if c == 0:
                     cmd.color('blue',oname)
                  elif c == 1:
                     cmd.color('white',oname)
                  else:
                     cmd.color('magenta',oname)
            c = c + 1
         save = cmd.set('auto_zoom',save,quiet=1)            
         if self.track<1:
            cmd.zoom("(pk1)",self.radius)
            cmd.clip("slab",self.radius-1)
         elif self.track<2:
            cmd.origin("(pk1)")
      cmd.refresh_wizard()      
# generic set routines

   def zoom(self):
      if 'pk1' in cmd.get_names('selections'):
         cmd.zoom("(pk1)",self.radius)
         cmd.clip("slab",self.radius-1)
      else:
         c = 0
         for a in self.map:
            oname = 'w'+str(c+1)+'_'+a
            if len(a):
               if a in cmd.get_names('objects'):
                  cmd.zoom(oname)
                  cmd.clip("slab",self.radius-1)
            c = c + 1
            
   def get_panel(self):
      self.update_map_menus()
      return [
         [ 1, 'Density Map Wizard',''],
         [ 2, 'Update Maps' , 'cmd.get_wizard().update_maps()'],
         [ 2, 'Zoom' , 'cmd.get_wizard().zoom()'],         
         [ 3, "Radius: %3.1f A"%self.radius,'radius'],
         [ 3, "Map 1: "+self.map[0],'map0'],
         [ 3, "       @ "+str(self.level[0])+" sigma",'level0'],
         [ 3, "Map 2: "+self.map[1],'map1'],
         [ 3, "       @ "+str(self.level[1])+" sigma",'level1'],
         [ 3, "Map 3: "+self.map[2],'map2'],
         [ 3, "       @ "+str(self.level[2])+" sigma",'level2'],
         [ 3, self.menu['track'][self.track][1], 'track' ],
         [ 2, 'Done','cmd.set_wizard()'],
         ]

   def cleanup(self):
      global default_radius,default_map,default_level
      default_radius = self.radius
      default_map = self.map
      default_level = self.level
      default_track = self.track
      self.clear()
      
   def clear(self):
      pass

   def do_pick(self,bondFlag):
      global dist_count
      if not bondFlag:
         if self.track<2:
            self.update_maps()
         



# pymol

from pymol import cmd
from chempy import io
from chempy import feedback
from chempy.bmin.state import State

import threading
import traceback
import os

state = None
model = None

def assign(sele,preserve=0):

   from molobj import MolObj
   from typer import Typer,Rules
   import rules

   result = 1
   
   state = State()

   model = cmd.get_model(sele)
   # now assign atom types

   ruleSet = Rules()

   ruleSet.fromList(rules.mmff_types)
   ruleSet.mappingFromList(rules.mmff_mapping)
   
   mobj = MolObj()
   mobj.fromChemPyModel(model)

   typed = Typer(molObj = mobj)
   
   print " realtime: assigning atom types"
   typed.applyRules(ruleSet)

   c = 0
   for a in typed.getTypes():
      at = model.atom[c]
      if (at.text_type == '??') or (not preserve):
         if a==-99:
            print " warning: unable to assign atom type to atom %d"%c
            result = 0
         else:
            cmd.alter("((%s) and (index %s))" % (sele,at.index),
                      "numeric_type ='%s'" % a)
            if feedback['tinker']:
               print " "+str(__name__)+': %s is a %s' % (at.name,a)
            at.numeric_type = a
      c = c + 1

   sm = 0
   for a in model.atom:
      a.resi = str(a.resi_number)
      sm = sm + a.partial_charge

   return result

def setup(sele,preserve=0):
   
   global state
   global model
   
   state = State()
   model = cmd.get_model(sele)

   sm = 0
   for a in model.atom:
      a.resi = str(a.resi_number)
      sm = sm + a.partial_charge

   state.load_model(model)
   return 1

def check(obj='check'):
   global state
   global model
   
   if not state:
      if not model:
         print " realtime.reload: please run setup first."
      else:
         cmd.load_model(model,obj,1)
   else:
      model = state.model
      cmd.load_model(model,obj,1)

def mini(total_steps=500,
         gradient=0.001,
         interval=100,
         object='rt',
         fix_flag=None,
         rest_flag=None,
         solvation=None,
         finish=None):

   global state
   if not state:
      print " realtime.mini: please run setup first..."
   else:
      model = state.model
      print " realtime.mini: %d atoms total\n" % model.nAtom
      try:
         while total_steps>0:
            total_steps = total_steps - interval
            state.minimize(max_iter=interval,
                           fix_flag=fix_flag,
                           rest_flag=rest_flag,
                           solvation=solvation)
            cmd.delete(object)
            cmd.load_model(state.model,object,1)
            cmd.refresh()
            if finish!=None:
               apply(finish[0],finish[1],finish[2])
      except:
         cmd.load_model(state.model,'ref')
         traceback.print_exc()
      print " realtime.mini: complete."

def mini_threaded(*args,**kwargs):
   t = threading.Thread(target=mini,
                        args=args,
                        kwargs=kwargs)
   t.setDaemon(1)
   t.start()
   







from chempy import Storage

class LST(Storage):

   def fromFile(self,fname,**params):
      fp = open(fname)
      result = fp.readlines()
      fp.close()
      return result

#---------------------------------------------------------------------------
   def toFile(self,list,fname,**params):
      fp = open(fname,'w')
      try:
         fp.writelines(list)
      except TypeError:
         for a in list:
            fp.write(str(a)+"\n")
      fp.close()
      





# filter wizard
# no-frills tool for quickly filtering docked compounds, etc.

import os,sys
from pymol.wizard import Wizard
from pymol import cmd
import pymol
import string
import traceback
# global dictionary for saving result on a per-object basis

static_dict = {}

# last/current object being filtered

default_object = None

# browing mode

default_browse = 1

accept_str = "Accept"
defer_str = "Defer"
reject_str = "Reject"

# class definition (class name must match wizard name with cap)

class Filter(Wizard):

   def __init__(self):

      # initialize parent class
      
      Wizard.__init__(self)
      
      # restore previous state from global storage

      self.dict = static_dict
      self.object = default_object
      self.browse = default_browse
      self.avail_objects = []
      self.state_dict = {}
      
      # if we don't have a current object, choose the first multi-state object
      
      if self.object == None:
         for a in cmd.get_names('objects'):
            if cmd.get_type(a)=='object:molecule':
               if cmd.count_states(a)>1:
                  self.object = a
                  break

      # menu for
      
      self.menu['browse'] = [
         [2, 'Browse Mode',''],
         [1, 'Browse All','cmd.get_wizard().set_browse(1)'],
         [1, 'Browse Accepted','cmd.get_wizard().set_browse(2)'],
         [1, 'Browse Rejected','cmd.get_wizard().set_browse(3)'],
         [1, 'Browse Deferred','cmd.get_wizard().set_browse(4)'],
         [1, 'Browse Remaining','cmd.get_wizard().set_browse(5)'],         
         ]

      self.count_object()
      self.load_state_dict()
      self.update_object_menu()
      cmd.set_key('F1',lambda s=self:s.accept())
      cmd.set_key('F2',lambda s=self:s.reject())
      cmd.set_key('F3',lambda s=self:s.defer())
      cmd.set_key('right',lambda s=self:s.forward())
      cmd.set_key('left',lambda s=self:s.backward())
      
   def update_object_menu(self):

      # find objects with > 1 state
      
      self.avail_objects = []
      
      for a in cmd.get_names('objects'):
         if cmd.get_type(a)=='object:molecule':
            if cmd.count_states(a)>1:
               self.avail_objects.append(a)

      # now create the object menu
      
      self.menu['object'] = [[2,'Select Object','']] 
      for a in self.avail_objects:
         self.menu['object'].append([ 1,a,'cmd.get_wizard().set_object("%s")'%(a) ])
      self.menu['object'].append([ 1,'None','cmd.get_wizard().set_object(None)'])
      
   def set_browse(self,browse):
      # allow user to focus on only a subset of the compounds
      self.browse = browse
      if self.browse == 1:
         print " Filter: Browsing all compounds."
         cmd.mset() # all states visible
      elif self.object==None:
         print " Filter-Error: please choose an object first"
      else: 
         self.check_object_dict()
         if self.browse == 2:
            print " Filter: Browsing accepted compounds."
            target = accept_str
         elif self.browse == 3:
            print " Filter: Browsing rejected compounds."            
            target = reject_str
         elif self.browse == 4:
            print " Filter: Browsing deferred compounds."                        
            target = defer_str
         lst = []
         sd = self.state_dict
         sdo = self.dict[self.object]
         if self.browse<5:
            for a in sdo.keys():
               if sdo[a]==target:
                  lst.append(sd[a])
         else:
            print " Filter: Browsing remaining compounds"
            for a in sd.keys():
               if not sdo.has_key(a):
                  lst.append(sd[a])
         lst.sort()
         if len(lst)==0:
            print " Filter-Error: No matching compounds."
         cmd.mset(string.join(map(str,lst),' '))
         cmd.rewind()
      cmd.refresh_wizard()

   def check_object_dict(self):
      # make sure we have a valid entry for this object in our dictionary
      
      if not self.dict.has_key(self.object):
         self.dict[self.object]={} # create dictionary to store results

   def adjust(self,decision,inc):
      # utility routine to increment/decrement counters 
      if decision == accept_str:
         self.acce = self.acce + inc
      elif decision == reject_str:
         self.reje = self.reje + inc
      elif decision == defer_str:
         self.defe = self.defe + inc

   def load_state_dict(self):
      # establish relationship between names and states
      # ASSUMPTION: identifiers will be unique
      self.state_dict = {}
      sd = self.state_dict
      so = self.object
      if so!=None:
         cnt = cmd.count_states(so)
         for a in range(1,cnt+1):
            sd[cmd.get_title(so,a)] = a

   def count_object(self):
      # record how many molecular are in an object, etc.
      self.check_object_dict()
      if self.object!=None:
         self.acce = 0
         self.reje = 0
         self.defe = 0
         self.togo = 0
         self.tota = cmd.count_states(self.object)
         sdo=self.dict[self.object]
         self.togo = self.tota-len(sdo)
         for a in sdo.keys():
            dec = sdo[a]
            self.adjust(dec,1)
      
   def set_object(self,obj_name):
      self.object = obj_name
      self.count_object()
      self.load_state_dict()
      cmd.refresh_wizard()
      
   def get_panel(self):

      # returns Wizard panel for PyMOL to display
      
      # 1 = title/text
      # 2 = button
      # 3 = pop-up menu
      
      self.update_object_menu()
      if self.object != None:
         save_str = 'Save %s.txt'%self.object
      else:
         save_str = ""
      return [
         [ 1, 'Filtering Wizard',''],
         [ 3, self.menu['browse'][self.browse][1], 'browse' ],
         [ 3, str(self.object), 'object' ],
         [ 2, 'Accept (F1)','cmd.get_wizard().accept()'],
         [ 2, 'Reject (F2)','cmd.get_wizard().reject()'],
         [ 2, 'Defer (F3)','cmd.get_wizard().defer()'],
         [ 2, 'Forward (->)','cmd.get_wizard().forward()'],                  
         [ 2, 'Back (<-)','cmd.get_wizard().backward()'],
         [ 2, save_str,'cmd.get_wizard().save()'],
         [ 2, 'Refresh','cmd.refresh_wizard()'],                  
         [ 2, 'Done','cmd.set_wizard()'],
         ]

   def get_prompt(self):

      # returns text prompt
      
      self.prompt = None
      if self.object == None:
         self.prompt = [ 'Please select a multi-state object...' ]
      else:
         cnt = cmd.count_states(self.object)
         self.prompt = [ '%s: %d total, %d accepted, %d rejected, %d deferred, %d remaining'%(
            self.object,self.tota,self.acce,self.reje,self.defe,self.togo) ]
         state = cmd.get_state()
         ident = cmd.get_title(self.object,state)
         sdo=self.dict[self.object]
         if sdo.has_key(ident):
            self.prompt.append('%s: %s'%(ident,sdo[ident]))
         else:
            self.prompt.append('%s?'%(ident))
      return self.prompt

   def count(self,entry,str):
      # keep track of how many compounds are in which category
      
      self.check_object_dict()
      sdo = self.dict[self.object]
      if sdo.has_key(entry):
         self.adjust(sdo[entry],-1)
      else:
         self.togo = self.togo - 1
      sdo[entry] = str
      self.adjust(sdo[entry],1)      
         
   def accept(self):
      # accept compound and advance
      if self.object==None:
         print " Filter-Error: Please choose an object first"
      else:
         state = cmd.get_state()
         ident = cmd.get_title(self.object,state)
         print " Filter: Accepting '%s'"%ident
         self.count(ident,accept_str)
      cmd.forward()         
      cmd.refresh_wizard()
      
   def reject(self):
      # reject compound and advance
      if self.object==None:
         print " Filter-Error: Please choose an object first"
      else:
         state = cmd.get_state()
         ident = cmd.get_title(self.object,state)
         print " Filter: Rejecting '%s'"%ident
         self.check_object_dict()
         self.count(ident,reject_str)
      cmd.forward()         
      cmd.refresh_wizard()
      
   def defer(self):
      # defer compound and advance
      if self.object==None:
         print " Filter-Error: Please choose an object first"
      else:
         state = cmd.get_state()
         ident = cmd.get_title(self.object,state)
         print " Filter: Deferring '%s'"%ident
         self.check_object_dict()
         self.count(ident,defer_str)
      cmd.forward()
      cmd.refresh_wizard()

   def forward(self):
      # go forward and update information
      cmd.forward()
      cmd.refresh_wizard()
      
   def backward(self):
      # go backward and update information      
      cmd.backward()
      cmd.refresh_wizard()
      
   def save(self):
      # write compounds to a file
      if self.object==None:
         print " Filter-Error: please choose an object first"
      else:
         self.check_object_dict()         
         fname = self.object+".txt"
         try:
            f=open(fname,'w')
            f.close()
         except:
            print " Filter-Warning: '"+fname+"' in current directory is not writable."
            print " Filter-Warning: attempting to write in home directory."
            if sys.platform[0:3]!="win":
               fname = "$HOME/"+fname
               fname = os.path.expandvars(fname)               
            else:
               fname = "$HOMEPATH\\"+fname
               fname = os.path.expandvars(fname)
               fname = "$HOMEDRIVE"+fname
               fname = os.path.expandvars(fname)
         try:
            f=open(fname,'w')
            sd = self.state_dict
            sdo = self.dict[self.object]
            f.write('Object\t"%s"\n'%(self.object))
            f.write('Total\t%d\nAccepted\t%d\nRejected\t%d\nDeferred\t%d\nRemaining\t%d\n\n'%(
               self.tota,
               self.acce,
               self.reje,
               self.defe,
               self.togo))
            # sort output in order of states            
            lst = []
            for a in sd.keys():
               lst.append((sd[a],a))
            lst.sort()
            # write list with decisions
            for a in lst:
               if sdo.has_key(a[1]):
                  f.write('%d\t"%s"\t"%s"\n'%(a[0],a[1],sdo[a[1]]))
               else:
                  f.write('%d\t"%s"\t"?"\n'%(a[0],a[1]))
            f.close()
            print " Filter: Wrote '%s'."%fname
         except:
            traceback.print_exc()
            print " Filter-Error: Unable to write '%s'."%fname
            
   def cleanup(self):
      # save current state in global vars...
      global default_object,default_browse,static_dict
      default_object = self.object
      default_browse = self.browse
      static_dict = self.dict
      # restore key actions
      cmd.set_key('F1',None)
      cmd.set_key('F2',None)
      cmd.set_key('F3',None)
      cmd.set_key('right',cmd.forward)
      cmd.set_key('left',cmd.backward)


         




from pymol.wizard import Wizard
from pymol import cmd
from pymol import util

import pymol

sele_pre = "sw_"
sele_prefix_len = len(sele_pre)

cent_sele = sele_pre+ "cent"
free_sele = sele_pre+ "free"
fix_sele = sele_pre+ "fix"
excl_sele = sele_pre+ "excl"

default_mode = 'by_resi'
default_radius = 6.0
default_cushion = 6.0

class Sculpting(Wizard):

   cutoff = 3.5
   
   def __init__(self):

      Wizard.__init__(self)
      
      self.status = 0 # 0 no atoms selections, 1 atom selected
      self.error = None
      self.object_name = None
      self.radius = default_radius
      self.cushion = default_cushion
      
      # mode selection subsystem
      
      self.mode = default_mode
      self.modes = [
         'ligand_rx',
#         'ligand_re',         
#         'by_atom',
         'by_resi',
         ]

      self.mode_name = {
         'by_atom':'Atom Shells',
         'by_resi':'Residue Shells',
         'ligand_rx' :'Residue v. Fixed',
         'ligand_re' :'Residue v. Free',                  
         'ligand_cx' :'Chain v. Fixed',
         'ligand_ce' :'Chain v. Free',                  
         }

      smm = []
      smm.append([ 2, 'Selection Mode', '' ])
      for a in self.modes:
         smm.append([ 1, self.mode_name[a], 'cmd.get_wizard().set_mode("'+a+'")'])
      self.menu['mode']=smm

      self.menu['radius'] = [[ 2, 'Mobile Atom Sphere', '' ],
                             [1, '4.0 A Radius','cmd.get_wizard().set_radius(4)'],
                             [1, '5.0 A Radius','cmd.get_wizard().set_radius(5)'],
                             [1, '6.0 A Radius','cmd.get_wizard().set_radius(6)'],
                             [1, '8.0 A Radius','cmd.get_wizard().set_radius(8)'],
                             [1, '10.0 A Radius','cmd.get_wizard().set_radius(10)'],
                             [1, '15.0 A Radius','cmd.get_wizard().set_radius(15)'],
                             [1, '20.0 A Radius','cmd.get_wizard().set_radius(20)'],
                             ]
                              
      self.menu['cushion'] = [[ 2, 'Fixed Atom Cushion', '' ],
         [1, '2.0 A Cushion','cmd.get_wizard().set_cushion(2)'],
         [1, '3.0 A Cushion','cmd.get_wizard().set_cushion(3)'],
         [1, '4.0 A Cushion','cmd.get_wizard().set_cushion(4)'],
         [1, '6.0 A Cushion','cmd.get_wizard().set_cushion(6)'],
         [1, '8.0 A Cushion','cmd.get_wizard().set_cushion(8)'],
         [1, '10.0 A Cushion','cmd.get_wizard().set_cushion(10)'],
         [1, '12.0 A Cushion','cmd.get_wizard().set_cushion(12)'],                              

         ]

# generic set routines

   def set_mode(self,mode):
      if mode in self.modes:
         self.mode = mode
      cmd.refresh_wizard()

   def set_radius(self,radius):
      self.radius=radius
      self.update_selections()
      cmd.refresh_wizard()
      
   def set_cushion(self,cushion):
      self.cushion=cushion
      self.update_selections()
      cmd.refresh_wizard()
      
   def update_selections(self):
      if self.status==1:
         if self.mode=='by_resi':
            cmd.select(free_sele,"((byobj %s) and byres ((%s) x; %8.3f))"%
                       (cent_sele,cent_sele,self.radius))
            cmd.select(fix_sele,"((byobj %s) and (not %s) and byres ((%s) x; %8.3f))"%
                       (cent_sele,free_sele,cent_sele,self.radius+self.cushion))
            cmd.select(excl_sele,"((byobj %s) and (not (%s|%s)))"%
                       (cent_sele,free_sele,fix_sele))
         elif self.mode=='ligand_rx':
            cmd.select(free_sele,"((byobj %s) and byres (%s))"%
                       (cent_sele,cent_sele))
            cmd.select(fix_sele,"((byobj %s) and (not %s) and byres ((%s) x; %8.3f))"%
                       (cent_sele,free_sele,free_sele,self.cushion))
            cmd.select(excl_sele,"((byobj %s) and (not (%s|%s)))"%
                       (cent_sele,free_sele,fix_sele))
         cmd.protect("(byobj %s)"%cent_sele)
         cmd.deprotect(free_sele)
         cmd.flag('exclude',"(byobj %s)"%cent_sele,"clear")
         cmd.flag('exclude',excl_sele,"set")
         cmd.color('grey',excl_sele)
         cmd.zoom(free_sele,3.0)
         util.cbac(fix_sele)
         util.cbag(free_sele)
         cmd.disable('indicate')
         cmd.disable(cent_sele)            
         cmd.disable(free_sele)
         cmd.disable(fix_sele)
         cmd.disable(excl_sele)
         cmd.unpick()
         
         self.status = 1
         
   def set_object_mode(self,mode):
      if mode in self.object_modes:
         self.object_mode = mode
      self.status = 0
      cmd.refresh_wizard()
      
   def get_panel(self):
      return [
         [ 1, 'Sculpting',''],
         [ 3, self.mode_name[self.mode],'mode'],
         [ 3, "Radius: %3.1f A"%self.radius,'radius'],
         [ 3, "Cushion: %3.1f A"%self.cushion,'cushion'],
         [ 2, 'Relocate','cmd.get_wizard().free_all()'],
         [ 2, 'Done','cmd.set_wizard()'],
         ]

   def free_all(self):
      if self.status==1:
         cmd.deprotect("(byobj %s)"%cent_sele)
         cmd.protect(fix_sele)
         cmd.flag('exclude',"(byobj %s)"%cent_sele,"clear")
         util.cbag("(byobj %s)"%cent_sele)
      self.clear()
      cmd.refresh_wizard()
      
   def clear(self):
      cmd.delete(sele_pre+"*")
      self.status = 0
      cmd.refresh_wizard()
      
   def cleanup(self):
      global default_mode, default_radius, default_cushion
      default_mode = self.mode
      default_radius = self.radius
      default_cushion = self.cushion
      self.clear()
      
   def get_prompt(self):
      self.prompt = None
      if self.status==0:
         self.prompt = [ 'Please pick the center atom...']
      if self.error!=None:
         self.prompt.append(self.error)
      return self.prompt
   
   def do_pick(self,bondFlag):
      global dist_count
      if not self.status:
         if cmd.select(cent_sele,"pk1"):
            self.status = 1
            self.update_selections()
         cmd.refresh_wizard()
         return 1
      else:
         return 0
      



#A* -------------------------------------------------------------------
#B* This file contains source code for the PyMOL computer program
#C* copyright 1998-2000 by Warren Lyford Delano of DeLano Scientific. 
#D* -------------------------------------------------------------------
#E* It is unlawful to modify or remove this copyright notice.
#F* -------------------------------------------------------------------
#G* Please see the accompanying LICENSE file for further information. 
#H* -------------------------------------------------------------------
#I* Additional authors of this source file include:
#-* 
#-* 
#-*
#Z* -------------------------------------------------------------------

# invocation.py
#
# This module unifies argument handling for embedded and modular PyMOL
#

import copy
import re
import os
import glob
import string

pattern1 = '.pymolrc*'
pattern2 = 'pymolrc*'

class generic:
   pass

options = generic();

options.stereo_capable = 0
options.deferred = []
options.no_gui = 0
options.internal_gui = 1
options.internal_feedback = 1
options.external_gui = 1
options.gui = 'pmg_tk'
options.show_splash = 1
options.read_stdin = 0
# Greg Landrum
options.rpcServer = 0

py_re = re.compile(r"\.py$|\.pym$\.PY$|\.PYM$")
pyc_re = re.compile(r"\.pyc$|\.PYC$")

def get_user_config():
   lst = glob.glob(pattern1)
   if not len(lst): # unix
      if os.environ.has_key("HOME"):
         lst = glob.glob(os.environ['HOME']+"/"+pattern1)
   if not len(lst): # unix
      if os.environ.has_key("HOME"):
         lst = glob.glob(os.environ['HOME']+"/"+pattern2)
   if not len(lst): # win32
      if os.environ.has_key("HOMEPATH") and os.environ.has_key("HOMEDRIVE"):
         lst = glob.glob(os.environ['HOMEDRIVE']+os.environ['HOMEPATH']+"/"+pattern1)
   if not len(lst): # win32
      if os.environ.has_key("HOMEPATH") and os.environ.has_key("HOMEDRIVE"):
         lst = glob.glob(os.environ['HOMEDRIVE']+os.environ['HOMEPATH']+"/"+pattern2)
   if not len(lst): # all
      if os.environ.has_key("PYMOL_PATH"):
         lst = glob.glob(os.environ['PYMOL_PATH']+"/"+pattern1)
   if not len(lst): # all
      if os.environ.has_key("PYMOL_PATH"):
         lst = glob.glob(os.environ['PYMOL_PATH']+"/"+pattern2)

   first = []
   second = []
   for a in lst:
      if py_re.search(a):
         first.append("_do__ run "+a) # preceeding "_ " cloaks 
      elif not pyc_re.search(a): # ignore compiled versions for now
         second.append("_do__ @"+a) # preceeding "_ " cloaks 
   first.sort()
   second.sort()
   return first+second

def parse_args(argv):
   av = copy.deepcopy(argv)
   av = av[1:] # throw out the executable path
   av.reverse()
   global options
   options.deferred = []
   # append user settings file as an option
   options.deferred.extend(get_user_config())
   while 1:
      if not len(av):
         break
      a = av.pop()
      a = re.sub(r'''^"|"$|^'|'$''','',a) # strip extra quotes
      if a[0:1]=='-':
         if a[1:2]=='-':
            break # double hypen signal end of PyMOL arguments
         if "c" in a:
            options.no_gui=1
            options.external_gui=0
         if "s" in a:
            pass # stereo now autodetected
         if "2" in a:
            options.deferred.append("_do__ config_mouse two_button")
         if "q" in a:
            options.show_splash = 0
         if "i" in a:
            options.internal_gui = 0
         if "f" in a:
            options.internal_feedback = int(av.pop())
         if "x" in a:
            options.external_gui = 0
         if "t" in a:
            options.gui = 'pmg_tk'
         if "w" in a:
            options.gui = 'pmg_wx'
         if "d" in a:
            options.deferred.append("_do_%s"%string.replace(av.pop(),'%',' '))
         if "e" in a:
            options.deferred.append("_do__ full_screen on")
         if "l" in a:
            options.deferred.append("_do_spawn %s"%av.pop())
         if "r" in a:
            options.deferred.append("_do_run %s,main"%av.pop())
         if "u" in a:
            options.deferred.append("_do_resume %s"%av.pop())
         if "s" in a:
            options.deferred.append("_do_log_open %s"%av.pop())
         if "p" in a:
            options.read_stdin = 1
         if "X" in a:
            options.rpcServer = 1
         if "g" in a:
            options.deferred.append("_do_png %s"%av.pop())
         
      else:
         options.deferred.append(a)
   if options.show_splash and not options.no_gui:
      png_path = os.path.expandvars("$PYMOL_PATH/modules/pymol/splash.png")
      if os.path.exists(png_path):
         options.deferred.insert(0,"_do__ cmd.load_png('%s',0,quiet=1)"%png_path)








import copy
import types
import re
import string

abbr_re = re.compile(r"[^\_]*\_")

def is_string(obj):
   return isinstance(obj,types.StringType)

def is_list(obj):
   return isinstance(obj,types.ListType)

class Shortcut:

   def __call__(self):
      return self
   
   def __init__(self,list,filter_leading_underscore=1):
      if filter_leading_underscore:
         self.keywords=filter(lambda x:x[:1]!='_',list)
      else:
         self.keywords = copy.deepcopy(list)
      self.shortcut = {}
      self.abbr_dict = {}
      self.rebuild()

   def add_one(self,a):
      # optimize symbols
      hash = self.shortcut
      abbr_dict = {}
      find = string.find
      abbr_re_sub = abbr_re.sub
      for b in range(1,len(a)):
         sub = a[0:b]
         if hash.has_key(sub):
            hash[sub]=0
         else:
            hash[sub]=a
      if find(a,"_")>=0:
         abbr = abbr_re_sub(lambda x:x.group(0)[0]+"_",a)
         if a!=abbr:
            if(abbr_dict.has_key(abbr)):
               abbr_dict[abbr].append(a)
            else:
               abbr_dict[abbr]=[a]
            for b in range(string.find(abbr,'_')+1,len(abbr)):
               sub = abbr[0:b]
               if hash.has_key(sub):
                  hash[sub]=0
               else:
                  hash[sub]=a
                  
         abbr = abbr_re_sub(lambda x:x.group(0)[0:2]+"_",a)
         if a!=abbr:
            if(abbr_dict.has_key(abbr)):
               abbr_dict[abbr].append(a)
            else:
               abbr_dict[abbr]=[a]
            for b in range(string.find(abbr,'_')+1,len(abbr)):
               sub = abbr[0:b]
               if hash.has_key(sub):
                  hash[sub]=0
               else:
                  hash[sub]=a
      
   def rebuild(self):
      # optimize symbols
      hash = self.shortcut
      self.abbr_dict = {}
      abbr_dict = self.abbr_dict
      self.abbr_list = []
      abbr_list = self.abbr_list
      find = string.find
      abbr_re_sub = abbr_re.sub
      #
      for a in self.keywords:
         for b in range(1,len(a)):
            sub = a[0:b]
            if hash.has_key(sub):
               hash[sub]=0
            else:
               hash[sub]=a
         if find(a,"_")>=0:
            abbr = abbr_re_sub(lambda x:x.group(0)[0]+"_",a)
            if a!=abbr:
               if(abbr_dict.has_key(abbr)):
                  abbr_dict[abbr].append(a)
               else:
                  abbr_dict[abbr]=[a]
               for b in range(string.find(abbr,'_')+1,len(abbr)):
                  sub = abbr[0:b]
                  if hash.has_key(sub):
                     hash[sub]=0
                  else:
                     hash[sub]=a
            abbr = abbr_re_sub(lambda x:x.group(0)[0:2]+"_",a)
            if a!=abbr:
               if(abbr_dict.has_key(abbr)):
                  abbr_dict[abbr].append(a)
               else:
                  abbr_dict[abbr]=[a]
               for b in range(string.find(abbr,'_')+1,len(abbr)):
                  sub = abbr[0:b]
                  if hash.has_key(sub):
                     hash[sub]=0
                  else:
                     hash[sub]=a
                     
      for a in abbr_dict.keys():
         adk = abbr_dict[a]
         if len(adk)==1:
            hash[a]=adk[0]
      for a in self.keywords:
         hash[a]=a

   def interpret(self,kee):
      if not len(kee): # empty string matches everything
         return copy.deepcopy(self.keywords)
      elif not self.shortcut.has_key(kee):
         return None # unrecognized, returns None
      elif self.shortcut[kee]==0:
         lst_dict = {}
         lcm = len(kee)
         for a in self.keywords:
            if a[0:lcm] == kee:
               lst_dict[a]=1 # ambiguous returns list
         for a in self.abbr_dict.keys():
            if a[0:lcm] == kee:
               for b in self.abbr_dict[a]:
                  lst_dict[b]=1
         lst = lst_dict.keys()
         if(len(lst)==1):
            return lst[0]
         elif not len(lst):
            return None
         return lst
      else:
         return self.shortcut[kee] # otherwise return string
      
   def has_key(self,kee):
      return self.shortcut.has_key(kee)

   def __getitem__(self,kee):
      if self.shortcut.has_key(kee):
         return self.shortcut[kee]
      else:
         return None

   def __delitem__(self,kee):
      self.keywords.remove(kee)
      self.rebuild()
      
   def append(self,kee):
      self.keywords.append(kee)
      self.add_one(kee)
      hash = self.shortcut      
      for a in self.abbr_dict.keys():
         adk = self.abbr_dict[a]
         if len(adk)==1:
            hash[a]=adk[0]
      for a in self.keywords:
         hash[a]=a

   def auto_err(self,kee,descrip=None):
      result = None
      if not self.shortcut.has_key(kee):
         if descrip!=None:
            print "Error: unknown %s: '%s'."%(
               descrip,kee),
            lst = self.interpret('')
            if is_list(lst):
               if len(lst)<100:
                  print "Choices:"
                  lst = parsing.list_to_str_list(lst)
                  for a in lst: print a
               else:
                  print
            else:
               print
            raise parsing.QuietException
         
      else:
         result = self.interpret(kee)
         if not is_string(result):
            if descrip!=None:
               print "Error: ambiguous %s:"%descrip
               lst = parsing.list_to_str_list(result)
               for a in lst:
                  print a
               raise parsing.QuietException
      return result
              
if __name__=='__main__':
   list = [ 'warren','wasteland','electric','well' ]
   sc = Shortcut(list)
   tv = sc.has_key('a')
   print tv==0,tv
   tv = sc.has_key('w')
   print tv==1,tv
   tv = sc.has_key('war')
   print tv==1,tv
   tv = sc.ambiguous('w')
   print tv==['warren', 'wasteland', 'well'],tv   
   tv = sc.ambiguous('e')
   print tv==None,tv   

import parsing



 #A* -------------------------------------------------------------------
#B* This file contains source code for the PyMOL computer program
#C* copyright 1998-2000 by Warren Lyford Delano of DeLano Scientific. 
#D* -------------------------------------------------------------------
#E* It is unlawful to modify or remove this copyright notice.
#F* -------------------------------------------------------------------
#G* Please see the accompanying LICENSE file for further information. 
#H* -------------------------------------------------------------------
#I* Additional authors of this source file include:
#-* 
#-* 
#-*
#Z* -------------------------------------------------------------------

import thread 
import threading 
import os
import sys
import re
import string 
import time
import invocation
import traceback
import math
import threading

# PyMOL __init__.py

# Create a temporary object "stored" in the PyMOL global namespace
# for usage with evaluate based-commands such as alter

class Scratch_Storage:
   pass

stored = Scratch_Storage()

# This global will be non-None if logging is active
# (global variable used for efficiency)

_log_file = None

# This global will be non-None if an external gui
# exists. It mainly exists so that events which occur
# in the Python thread can be handed off to the
# external GUI thread through one or more FIFO Queues
# (global variable used for efficiency)

_ext_gui = None

# include the modules directory

modules_path = os.environ['PYMOL_PATH']+'/modules'
if modules_path not in sys.path:
   sys.path.append(modules_path)

# include installed numpy on win32 

if sys.platform=='win32':
   sys.path.append(os.environ['PYMOL_PATH']+'/modules/numeric')

sys.setcheckinterval(1) # maximize responsiveness

lock_api = threading.RLock() # mutex for API 
lock_api_c = threading.RLock() # mutex for C management of python threads

def start_pymol():
   global glutThread
   glutThread = thread.get_ident()
   _cmd.runpymol() # only returns if we are running pretend GLUT
   from pymol import wxpymol # never returns

def exec_str(s):
   try:
      exec s in globals(),globals()
   except StandardError:
      traceback.print_exc()
   return None

def stdin_reader(): # dedicated thread for reading standard input
	import sys
	from pymol import cmd
	while 1:
		cmd.do(sys.stdin.readline())

			   
def exec_deferred():
   
   try:
      cmd.config_mouse(quiet=1)
      for a in invocation.options.deferred:
         if a[0:4]=="_do_":
            cmd.do(a[4:])
         elif re.search(r"pymol\.py$",a):
            pass
         elif re.search(r"\.py$|\.pym|\.pyc$",a,re.I):
            cmd.do("_ run %s" % a)
         elif cmd.file_ext_re.search(a):
            cmd.load(a)
         elif re.search(r"\.pml$",a,re.I):
            cmd.do("_ @%s" % a)
         else:
            cmd.load(a)
   except:
      traceback.print_exc()
   if invocation.options.read_stdin:
      t = threading.Thread(target=stdin_reader)
      t.setDaemon(1)
      t.start()

def adapt_to_hardware():
   (vendor,renderer,version) = cmd.get_renderer()
   if vendor[0:6]=='NVIDIA':
      if renderer[0:7]=='GeForce':
         if invocation.options.show_splash:
            print " Adapting to GeForce hardware..."
         cmd.set('line_width','2',quiet=1)
      elif renderer=='NVIDIA GPU OpenGL Engine':
         if sys.platform=='darwin':
            if invocation.options.show_splash:
               print " Adapting to NVIDIA hardware on Mac..."
               cmd.set('line_smooth',0,quiet=1)
               cmd.set('fog',0.9,quiet=1)

         
# NEED SOME CONTRIBUTIONS HERE!

def launch_gui():
   if invocation.options.external_gui:
      __import__(invocation.options.gui)

# -- Greg Landrum's RPC stuff
   if invocation.options.rpcServer:
      import rpc
      rpc.launch_XMLRPC()
# --

import _cmd
import cmd

if os.environ.has_key('DISPLAY'):
   from xwin import *







from chempy.models import Indexed
from chempy import Storage,Atom,Bond

import string

class MOL(Storage):

   def fromList(self,molList):

      model = Indexed()

      # read header information
      model.molecule.title = string.strip(molList[0])
      model.molecule.dim_code = string.strip(molList[1][20:22])
      model.molecule.comments = string.strip(molList[2])
      model.molecule.chiral = int(molList[3][12:15])
      nAtom = int(molList[3][0:3])
      nBond = int(molList[3][3:6])

      # read atoms
      nameDict = {}
      irec = 4
      cnt = 0
      for a in range(nAtom):
         at = Atom()
         at.index = cnt
         at.coord = [float(molList[irec][0:10]), 
            float(molList[irec][10:20]),float(molList[irec][20:30])]
         at.symbol = string.strip(molList[irec][31:33])
         at.stereo = int(molList[irec][39:42])         
         chg=int(molList[irec][36:39])
         if chg>0: chg=4-chg
         at.formal_charge = chg
         model.atom.append(at)
         irec = irec + 1
         cnt = cnt + 1

         # read bonds
      for a in range(nBond):
         bnd = Bond()
         bnd.index = [ int(molList[irec][0:3])-1,int(molList[irec][3:6])-1 ]
         bnd.order = int(molList[irec][6:9])
         bnd.stereo = int(molList[irec][9:12])
         model.bond.append(bnd)
         irec = irec+1

         # obtain formal charges from M  CHG record
      while molList[irec][0:6]!='M  END':
         if molList[irec][0:6]=='M  CHG':
            cl = string.split(string.strip(molList[irec][6:]))
            cll = int(cl[0])*2
            a=1
            while a<=cll:
               model.atom[int(cl[a])-1].formal_charge=int(cl[a+1])
               a=a+2
         irec =irec+1
         if irec >= len(molList): break

      return model

#------------------------------------------------------------------------------
   def toList(self,model):

      molList = []

      # write header records
      molList.append(model.molecule.title+"\n")
      molList.append("  ChemPy            %2s                             0\n" %
             model.molecule.dim_code)
      molList.append(model.molecule.comments+"\n")
      molList.append("%3d%3d  0  0  %1d  0  0  0  0  0999 V2000\n" %
                     (model.nAtom, model.nBond, model.molecule.chiral))

      # write atom records
      for a in model.atom:
         chg = a.formal_charge
         if chg!=0: chg=4-chg
         molList.append("%10.4f%10.4f%10.4f %-3s 0  %1d  %1d  0  0  0  0  0  0  0  0  0\n" % \
                     (a.coord[0], a.coord[1], a.coord[2], a.symbol, chg, a.stereo))

         # write bond records
      for b in model.bond:
         molList.append("%3d%3d%3d%3d  0  0  0\n" % (b.index[0]+1, 
            b.index[1]+1, b.order,b.stereo))

         # if necessary, write M  CHG records for charged atoms
      charge_atoms = []
      charge_values = []
      for a in model.atom:
         if a.formal_charge != 0:
            charge_atoms.append(a)
      while len(charge_atoms) != 0:
         chg_set = charge_atoms[0:8]
         charge_atoms = charge_atoms[8:]
         tline = "M  CHG%3d" % (len(chg_set))
         for i in chg_set:
            tline = tline + "%4d%4d" % (i.index+1,i.formal_charge)
         molList.append(tline + "\n")
      molList.append("M  END\n")
      return(molList)





# commands extensions to PyMOL for batchmin

from pymol import cmd
from chempy.bmin import realtime
import threading

def amin(*arg,**kwarg):
   realtime.assign(arg[0])
   apply(bmin,arg,kwarg)

def bmin(object,iter=500,grad=0.1,interval=100,
         solvation=None):
   realtime.setup(object)
   t = threading.Thread(target=realtime.mini,
                        args=(int(iter),float(grad),int(interval),str(object)),
                        kwargs={
      'solvation':solvation, # turn on GB/SA (cdie)
      'rest_flag':2, # atoms with flag 2 set are restrained
      'fix_flag':3, # atoms with flag 3 set are fixed
      }
                        )
   t.setDaemon(1)
   t.start()

def bmin_sync(object,iter=500,grad=0.1,interval=100,
         solvation=None):
   realtime.setup(object)
   realtime.mini(int(iter),float(grad),int(interval),str(object),
                 solvation=solvation,
                 rest_flag=2,
                 fix_flag=3)
   
def pmin():
   cmd.delete('min')
   realtime.assign('lig')
   cmd.create('min','(lig|prot)')
   bmin('min')
   
cmd.extend('bmin',bmin)
cmd.extend('bmin_sync',bmin_sync)
cmd.extend('amin',amin)
cmd.extend('pmin',pmin)

